.module       fft_sub;
.pagelength   59;

{************************************************************************
 *
 *          EZ-KIT Lite DSP Program for DSP-10 2-meter Transciever
 *			F2N1024B.DSP Version 1.60
 *		   FFT, Squaring to power and Log Module
 *
 * NOTICE: Copyright (C) 1996, 1997, 1998, 1999, 2004 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm
 *
 * The fft routine is heavily based on those in Analog Devices Applications
 * Using the ADSP 2100 Family, Vol 1.
 *
 ****************************************************************************}

.const		LOG2N=10, N=1024;       {Set constants for N-point FFT}
.const		N_DIV_2=512;
.const		N_TIMES_2=2048;
.const		mod_value=H#0010;	{ mod_value=16384/N }
.const		scrambletop=H#0000;	{ Here, ^inplacereal=h#0000 which is }
					{ the same as its bit-reversed address}
.const		log_2=H#2688;

.external       twid_real, twid_imag;
.external	dataptr;		{ ptr to N-point seq input array }
.external       inplacereal, inplaceimag;
.external       groups, bflys_per_group, node_space;
		{ The following three are N_DIV_2 arrays:  }
.external	out_log;                   { 8 bit log for serial trans }
.external	log_offset;
.external	power_exp;
.external	power_hi, power_med, power_lo;   { Power averages }
.external	integrate_count;           { 4 down to 0 }
.external	blk_exponent;
.external	mode_select;
.external	status_buf;
.external	tx_buf;
.var/pm		log_coeffs[5];
.init		log_coeffs : H#6F1500, H#C98700, H#1FC300, H#F13500, H#037D00;
.var/dm		i5_sav;
.external	mode_select;
.var/dm		squelch_noise;		{ Est of squelch noise power }
.global		squelch_noise;
.var/dm		t1, t2;
.external	data_format;
.var/dm		sqh, sql;
.external	do_v2f, v2f_bin_ready;
.external	v2f_bin_low, v2f_bin_high;
.external	v2f_pwrl, v2f_pwrh, v2f_pkl, v2f_pkh;
.entry  	scramble;
.entry		log_power;
.entry		log10_dec;
.entry		log10;
.entry          fft_strt;
.entry		dump_intrgrt;

{********************************  SCRAMBLE  ********************************
 *     Calling Parameters
 *              Sequentially ordered input data in *inptr
 *      Return Values
 *              Scrambled input data in inplacereal
 *
 *	Note that if DAG1 is used by an interrupt routine during scrambling
 *	the addresses of arrays in the  ISR will be scrambled!
 *	Thus the short disabling of interrupts.
 *
 ****************************************************************************}
scramble:
	i4=dm(dataptr);		{ i4-->sequentially ordered data}
       	i0=scrambletop;		{ i0-->scrambled data}
 	m4=1;
       	m0=mod_value;   	{ m0=modifier for reversing N bits}
       	l4=0;
       	l0=0;
       	cntr=N;
	{ The following is done to allow SPORT0 interrupts to be on time.  }
	do brev until ce;
	  ay1=dm(i4, m4);		{ Note: interrupt allowed here }
	  imask=0;			{ Disable interrupts, each time! }
	  ena bit_rev;			{ Bit reversed DAG1 }
	  dm(i0, m0)=ay1;
	  dis bit_rev;
brev:	  imask=b#0000110000;		{ Re-enable SPORT0 Rcvr interrupt }
					{   and IRQE }
       	rts;

{***************************************************************************
 *
 *     FFT_STRT()
 *
 *     Performs Radix-2 DIT FFT and convert to power and accumulates
 *     in 48 bit sums, power_lo, power_med, and power_hi.
 *
 *      Calling Parameters
 *              inplacereal = Real input data in scrambled order
 *              inplaceimag = All zeroes (real input assumed)
 *              twid_real = Twiddle factor cosine values
 *              twid_imag = Twiddle factor sine values
 *              groups = N/2
 *              bflys_per_group = 1
 *              node_space = 1
 *
 *      Return Values
 *              inplacereal = Real FFT results in sequential order      
 *              inplaceimag = Imaginary FFT results in sequential order 
 *		power_lo    = 512 array of low 16 bits of integrated power
 *		power_med   = 512 array of middle 16 bits
 *		power_hi    = 512 array of high 8 bits
 *                              
 ****************************************************************************}

fft_strt:
	ax0 = N_DIV_2;                  { Initialize fft variables }
	dm(groups) = ax0;
	ax0 = 1;
	dm(bflys_per_group) = ax0;
	dm(node_space) = ax0;
	ax0 = 0;
	dm(blk_exponent) = ax0;
	i0 = ^inplaceimag;  m0 = 1;  l0 = 0;
        cntr = N;
        do zero until CE;
zero:	    dm(i0, m0) = ax0;
	CNTR=LOG2N;     {Initialize stage counter}
        M0=0;  M1=1;
	L0=0;  L1=0;  L2=0;  L3=0;
        L4=%twid_real;  L5=%twid_imag;

	{ The fft will grow 1 bit per stage on big sine waves, and if phases
	  are completely to real or imaginary they can grow 2 bits.  To prevent
	  overload on first stage we need to make sure that the biggest
	  magnitude is not over 14 bits:          }
	I0=^inplacereal;
	sb=-2;				{ Check for >14 bits of input }
	cntr=N;
	do chk14 until ce;
	  ar=dm(i0, m1);
	  sb=expadj ar;			{ Check for input magnitude }
	chk14:  nop;
	call bfp_adj1;			{ Compensate for >14 bit input }
        DO stage_loop UNTIL CE;   {Compute all stages in FFT}
	    { CNTR takes values LOG2N, LOG2N-1, ...,1 }
            I0=^inplacereal;        {I0 -->x0 in 1st grp of stage}
            I2=^inplaceimag;        {I2 -->y0 in 1st grp of stage}
            SB=-2;                  {SB to detect data > 14 bits}
            SI=DM(groups);
            CNTR=SI;                {CNTR = group counter}
            M4=SI;                  {M4=twiddle factor modifier}
            M2=DM(node_space);      {M2=node space modifier}
            I1=I0;
            MODIFY(I1,M2);          {I1 -->y0 of 1st grp in stage}
            I3=I2;
            MODIFY(I3,M2);          {I3 -->y1 of 1st grp in stage}
            DO group_loop UNTIL CE;
              I4=^twid_real;                    {I4 --> C of W0}
              I5=^twid_imag;                    {I5 --> (-S) of W0}
              CNTR=DM(bflys_per_group);         {CNTR = butterfly counter}
              MY0=PM(I4,M4),MX0=DM(I1,M0);      {MY0=C,MX0=x1 }
              MY1=PM(I5,M4),MX1=DM(I3,M0);      {MY1=-S,MX1=y1}
              DO bfly_loop UNTIL CE;
                MR=MX0*MY1(SS),AX0=DM(I0,M0);   {MR=x1(-S),AX0=x0}
                MR=MR+MX1*MY0(RND),AX1=DM(I2,M0);
                                                {MR=(y1(C)+x1(-S)),AX1=y0}
                AY1=MR1,MR=MX0*MY0(SS);         {AY1=y1(C)+x1(-S),MR=x1(C)}
                MR=MR-MX1*MY1(RND);             {MR=x1(C)-y1(-S)}
                AY0=MR1,AR=AX1-AY1;             {AY0=x1(C)-y1(-S),}
                                                {AR=y0-[y1(C)+x1(-S)]}
                SB=EXPADJ AR,DM(I3,M1)=AR;      {Check for bit growth,}
                                                {y1=y0-[y1(C)+x1(-S)]}
                AR=AX0-AY0,MX1=DM(I3,M0),MY1=PM(I5,M4);
                                                {AR=x0-[x1(C)-y1(-S)],}
                                                {MX1=next y1,MY1=next (-S)}
                SB=EXPADJ AR,DM(I1,M1)=AR;      {Check for bit growth,}
                                                {x1=x0-[x1(C)-y1(-S)]}
                AR=AX0+AY0,MX0=DM(I1,M0),MY0=PM(I4,M4);
                                                {AR=x0+[x1(C)-y1(-S)],}
                                                {MX0=next x1,MY0=next C}
                SB=EXPADJ AR,DM(I0,M1)=AR;      {Check for bit growth,}
                                                {x0=x0+[x1(C)-y1(-S)]}
                AR=AX1+AY1;                     {AR=y0+[y1(C)+x1(-S)]}
bfly_loop:      SB=EXPADJ AR,DM(I2,M1)=AR;      {Check for bit growth,}
                                                {y0=y0+[y1(C)+x1(-S)]}
              MODIFY(I0,M2);  {I0 -->1st x0 in next group}
              MODIFY(I1,M2);  {I1 -->1st x1 in next group}
              MODIFY(I2,M2);  {I2 -->1st y0 in next group}
group_loop:   MODIFY(I3,M2);  {I3 -->1st y1 in next group}

            call bfp_adj;		{ Compensate for bit growth }
            SI=DM(bflys_per_group);
            SR=ASHIFT SI BY 1(LO);
            DM(node_space)=SR0;         {node_space=node_space*2}
            DM(bflys_per_group)=SR0;    {bflys_per_group=bflys_per_group*2}
            SI=DM(groups);
            SR=ASHIFT SI BY -1(LO);
stage_loop: DM(groups)=SR0;             {groups=groups/2}

	 {Power spectrum calculation.  Find sum of squares of real and
	  imag components.  Two voltages (-16384, 16383) corresponding
	  to one bit growth in the final stage,
	  are squared and added to produce a 30 bit result.
	  This is added to power_lo, _med and _hi for non-coherent integration.
	  Correction is made for blk_exponent by right shift of 12-2*b_e.
	  If b_e is 0 there is a right shift of 12, and if it is, say 5,
	  we right shift shift the power answers by 12-2*5=2 places.
	  This keeps binary points of the power data lined up.     }
        i0=^inplaceimag; l0=0;		{ Start at begining of spectrum }
        i1=^inplacereal; l1=0;
        i3=^power_lo;    l3=0;
	m0=0;  m1=1;
	i4=^power_med;   l4=0;
        i5=^power_hi;    l5=0;
	m4=0;  m5=1;
	ay0=dm(blk_exponent);
	ax0=ay0;
	ar=ax0+ay0;			{ Double shift amount }
	ay0=dm(power_exp);
	ar=ar-ay0;			{ 2*be-12 }
	se=ar;				{ To control amount of shift }

        cntr=N_DIV_2;
	do pwr_lp until ce;
            mx0=dm(i0,m1);		{ Get a quadrature voltage }
            mr=mx0*mx0(ss);		{ Partial power }
            mx1=dm(i1,m1);		{ Get a in-phase voltage }
            mr=mr+mx1*mx1(ss);		{ Power estimate,  >0 }

	    { The next operation is not ashift since powers are positive
		and we want zero fill from left for 32 bit power }
	    sr=lshift mr1 (hi);		{ Correct for blk_exponent }
	    sr=sr OR lshift mr0 (lo);	{ Finish the correction }

	    ay0=dm(i3, m0);  ar=sr0+ay0;  { Accumulate power (48 bit) }
	    dm(i3, m1)=ar;
	    ay0=dm(i4, m4);  ar=sr1+ay0+c;
	    dm(i4, m5)=ar;
	    ax0=dm(i5, m4); ay0=0;  ar=ax0+ay0+c; { Add in carry }
pwr_lp:	    dm(i5, m5)=ar;

	{ If in FM, create squelch level }
	ax0=dm(mode_select);
	ay0=2;				{ See if we are receiving fm }
	none=ax0-ay0;
	if ne jump non_fm;		{ Not FM, so skip over }

	{ Find squelch noise by integrating power from 2500 to 3200 Hz,
	  or bins 267 to 341. This is done for each fft, every 1/18.75 sec.

	  The fft has scaled outputs by 2^blk_exponent.  To use the data
	  for fm squelch we need to unscale.  Since this has been squared
	  to power we need to set up a shift of 2*blk_exponent places:   }
	sr1=ar;
	ax0=dm(blk_exponent);		{ Now scale by blk_exponent }
	ay0=7;
	ar=ax0-ay0;
	ay0=ar;				{ Now x2 }
	ar=ar+ay0;
	se=ar;

	{ Now index into the fft output data: }
	i0=^inplaceimag+267;
	i1=^inplacereal+267;
	m0=1;
        cntr=74;			{ 341-267 }
	do sq_pwr until ce;
            mx0=dm(i0,m0);
            mr=mx0*mx0(ss);		{ Real squared }
            mx1=dm(i1,m0);
            mr=mr+mx1*mx1(ss);		{ Real sq + Imag sq }

	    sr = lshift mr1 (hi);	{ Adjust for blk_exponent }
	    sr = sr or lshift mr0 (lo);

	    ay0=dm(sql);  ar=sr0+ay0;  { Accumulate power (32 bit) }
	    dm(sql)=ar;
	    ay0=dm(sqh);  ar=sr1+ay0+c;
sq_pwr:	    dm(sqh)=ar;

	sr = lshift ar by 4 (hi);	{ Create 16 bit power that is valid }
	mr0=dm(sql);			{    with weaker signals  }
	sr = sr or lshift mr0 by 4 (lo);  { It is high 16 bits }
	{ WAS by 5 before ver 2.3 }
	{ Now RC filter the noise power: }
	my0=6554;			{ 20% new noise }
	mr=sr1*my0 (ss);
	mx0=dm(squelch_noise);  my0=26215;
	mr=mr+mx0*my0 (ss);		{ Add in 80% of old noise }
	dm(squelch_noise)=mr1;
	ax0=0;  dm(sql)=ax0;  dm(sqh)=ax0;	{ Zero pwr sums }

	{ See if v2f is being done from bin powers }
non_fm:	ax0=dm(do_v2f);
	ar=tstbit 1 of ax0;
	if eq rts;			{ 0=WB Noise, 1=bin powers }

	{ This process runs at a 18.75 Hz rate. The interrupt driven process
	  for updating the v2f tone runs at 9600 Hz. So we only need to 
	  update the tone every 512 of those. This happens by the variable
	  dm(v2f_bin_ready) that is 1 after new data is added here, but set to
	  0 in the ISR.  We don't check it here at entry, as it will always be
	  set to zero, long before the new 512 data points are collected.  }

	{ The V2F function can run off of wide band noise, or from specified
	  FFT bins. These latter can be either averaged or peak detected. Here
	  we scan the bins v2f_bin_low to _high and put the sum into v2f_pwrl
	  and v2f_pwrh and the max power into v2f_pkl and v2f_pkh.
	  First index into the fft output data: }
	ax0=^inplaceimag;
	ay0=dm(v2f_bin_low);
	ar=ax0+ay0;  i0=ar;

	ax0=^inplacereal;		{ ay0 is still dm(v2f_low) }
	ar=ax0+ay0;  i1=ar;

        ax0=dm(v2f_bin_high);		{ ay0 is still dm(v2f_low) }
	ar=ax0-ay0;  cntr=ar;
	m0=1;

	ax0=0;				{ Zero ave power and peak power }
	dm(v2f_pwrl)=ax0;  dm(v2f_pwrh)=ax0;
	dm(v2f_pkl)=ax0;  dm(v2f_pkh)=ax0;

	do v2f_pwr until ce;
            mx0=dm(i0,m0);
            mr=mx0*mx0(ss);		{ Real squared }
            mx1=dm(i1,m0);
            mr=mr+mx1*mx1(ss);		{ Real sq + Imag sq }

	    sr = lshift mr1 (hi);	{ Adjust for blk_exponent }
	    sr = sr or lshift mr0 (lo);

	    ay0=dm(v2f_pwrl);  ar=sr0+ay0;  { Accumulate power (32 bit) }
	    dm(v2f_pwrl)=ar;
	    ay0=dm(v2f_pwrh);  ar=sr1+ay0+c;
	    dm(v2f_pwrh)=ar;

	    { See if latest bin power is the new peak power }
	    ay0=dm(v2f_pkl); ar=sr0-ay0;	{ Check low 16 bits }
	    ay0=dm(v2f_pkh); ar=sr1-ay0+c-1;	{ Check high 16 bits }
	    if lt jump v2f_pwr;			{ Jump if old peak was highest}
	    dm(v2f_pkl)=sr0;  dm(v2f_pkh)=sr1;	{ Get new peak }
v2f_pwr:    nop;
	ax0=1; dm(v2f_bin_ready)=ax0;		{ To only do process as ready }
	rts;

{******************  BLOCK FLOATING POINT ADJUST  *************************
 *	Called by fft to scale arrays to prevent overflow.
 *	Calling Parameters
 *               Radix-2 DIT FFT stage results in inplacereal and inplaceimag
 *      NOTE: this code assumes inplaceimag immediately follows
 *	inplacereal in memory!
 *
 *	Return Parameters
 *		inplacereal and inplaceimag adjusted for bit growth
 *		blk_exponent   The number of shifts used total.
 *
 ***************************************************************************}
bfp_adj:    AY0=CNTR;                   {Check for last stage}
            AR=AY0-1;
            IF EQ RTS;                  {If last stage, return}
bfp_adj1:   AY0=-2;			{ Alternate entry for input check }
            AX0=SB;
            AR=AX0-AY0;                 {Check for SB=-2}
            IF EQ RTS;			{IF SB=-2, no bit growth, return}

            I0=^inplacereal;            {I0=read pointer}
            I1=^inplacereal; M1=1;      {I1=write pointer}
            AY0=-1;
            MY0=H#4000;                 {Set MY0 to shift 1 bit right}
            AR=AX0-AY0,MX0=DM(I0,M1);   {Check if SB=-1; Get first sample}
            IF EQ JUMP strt_shift;      {If SB=-1, shift block data 1 bit}

            AY0=-2; 	                {Set AY0 for block exponent update}
            MY0=H#2000;                 {Set MY0 to shift 2 bits right}
strt_shift: CNTR=N_TIMES_2 - 1;         {initialize loop counter}
            DO shift_loop UNTIL CE;     {Shift block of data}
              MR=MX0*MY0(RND),MX0=DM(I0,M1);  {MR=shifted data,MX0=next value}
shift_loop:   DM(I1,M1)=MR1;          	      {Unshifted data=shifted data}
            MR=MX0*MY0(RND);            { Shift last data word}
            AY0=DM(blk_exponent);       { Update block exponent and}
            DM(I1,M1)=MR1,AR=AY0-AX0;   {  store last shifted sample}
            DM(blk_exponent)=AR;
	    RTS;

{******************************  LOG_POWER()  *****************************
 * This creates the 8 bit log from power_hi, power_med and power_lo
 ************************************************************************}
log_power:
	i0=^power_lo;	l0=0;
	i1=^power_med;	l1=0;
	i2=^power_hi;	l2=0;
	m0=0;  m1=1;
        i3=^out_log;	l3=0;

	cntr=N_DIV_2;
	do lp1 until ce;
            mr0=dm(i0, m0);		{ Get true power mantissa, lo }
            mr1=dm(i1, m0);		{   and med. Leave pointers for now }
	    mr2=dm(i2, m0);		{   and hi.  }
            call log10;			{ Log10 in 4.12 in sr1 }
	    ax0=dm(data_format);	{ Are we 8 or 16 bit? }
	    none = pass ax0;
	    if gt jump lp0;		{ Skip scaling if 16 bit }
            ay0 = dm(log_offset);	{ Offset to center 8 bit log data }
            ar = sr1+ay0;
	    if av jump lp2;		{ Sum exceeded 32767 }
            if ge jump lp00;            { Don't allow 8 bit log <0 }
	    ar = 0;
	    { Log10 provides 4096 per decade. For 8 bit data we want 80 dB
		to occupy 256 or 32 per decade.  4096 to 32 requires
		a shift of 7 places.				}
lp00:	    sr = ashift ar by -7 (hi);	{ Div by 128 to create 8 bit log }
lp0:	    dm(i3, m1) = sr1;		{ Store log result in out_log }

            ax0 = 0;
	    dm(i0, m1) = ax0;		{ Dump integrator: power_lo }
	    dm(i1, m1) = ax0;		{   & power_med   }
lp1:	    dm(i2, m1) = ax0;		{   & power_hi and increment index }
	rts;

lp2:	ar=32767;		{ For overflow, use max positive value }
	jump lp00;		{ Return to program flow }

{******************************  LOG10()  ******************
 * This creates the 16 bit dB level from power in mr.
 * Input:	mr	power 1 to  2^39 - 1
 *			(ref=2^16  (65536) )
 * that is,     mr2, mr1 = integer portion of input in 24.0 twos complement
 *              mr0 = fractional portion of input in 0.16 unsigned
 * Output:	sr1 = log base 10 in 4.12 format
 *
 * This is basically taken from chapter 4 of applications handbook volume 1.
 * Modified to take a 40 bit input in mr2, mr1, mr0.   RSL 11 Dec 97
 * Defined for all input values > 0.
 *
 ************************************************************************}
log10:	m4=1; l4=0;
        i4=^log_coeffs;
        my1=LOG_2;
	{ We can have a 40 bit input in mr needing log10().
	  mr2 is maximum of 8 bits.  First we prescale the mr to make
	  mr2=0 and mr1 having sign bit=0 (a positive number):           }
	se = exp mr2 (hi);		{ Find shift needed to normalize }
	se = exp mr1 (lo);
	ax0 = se;			{ Find complement16 to se }
	ar = ax0 +16;
	ay1 = ar;			{ This gets added onto the
					  multiplier for log10(2) }
	sr = norm mr2 (hi);
	sr = sr or norm mr1 (lo);
	{ mr2 and mr1 are now shifted into sr.  mr0 requires an opposite
	  shift to align it for or'ing into sr: }
	se = ay1;			{ Number of shifts + 1 }
	sr =sr or norm mr0 (lo);
	mr1 = sr1;
	mr0 = sr0;
	{ Now ready for regular log routine from AD }
        SE=EXP MR1 (HI);		{ Count number of redundant bits}
        SE=EXP MR0 (LO);
        AY0=SE;				{ From se, calc number of log10(2) }
        AR=AY0-1;
        AX0=14;
        SE=AR, AR=AX0+AY0;
        SR=NORM MR1 (HI);		{ Remove redundant bits}
        SR=SR OR NORM MR0 (LO);
	ar = ar + ay1;			{ Addition for mr2 }
	af = ar + 32;			{ ar=-32 for input of mr=0 }
	if eq jump log0;		{ Out=-inf or -32768 }
        MR=AR*MY1 (SS);			{ (SE+14) * log10(2) }
        MY1=MR1;			{ Shift left 16 bits}
        MR1=MR0;
        MR2=MY1;
        MR0=0;
        AY0=H#8000;			{ Use x-1 in power series }
        AR=SR1-AY0;			{ Subtract 1 }
        MY1=AR;				{ Calculate power series }
        MF=AR*MY1 (RND), MX1=pm(i4,m4);  {MF = x**2}
        MR=MR+MX1*MY1 (SS), MX1=PM(i4,m4); {MR = C1*x}
        CNTR=3;
log_aprx:    MR=MR+MX1*MF (SS);
             MF=AR*MF (RND), MX1=PM(i4,m4);
        if not ce jump log_aprx;	{ Test & post-decrement }
        MR=MR+MX1*MF (RND);
        SR=ASHIFT MR2 BY 12 (HI);           {Shift to correct format}
        SR=SR OR LSHIFT MR1 BY 12 (LO);
	{ Now have sr1 = Log base 10 in 4.12 format.
	  The smallest input, 1, produces an output sr1=B2F0 (-19728).
	  The largest input, 7F FFFF FFFF produces  sr1=6EC7 (28359).
	  Each decade produces an increase of 4096 (2^12). Res=1/409.6 dB
	  Factors of two produce an increase of 1233.
	  Unity input, FFFF, produces 0 output.  }
	rts;
log0:	sr1=-32768;
	rts;

{******************************  LOG10_DEC()  ****************************
 * This creates the 16 bit dB level from power in mr.
 * Input:	mr	power 1 to  2^39 - 1
 *			(ref=2^16  (65536) )
 * that is,     mr2, mr1 = integer portion of input in 24.0 twos complement
 *              mr0 = fractional portion of input in 0.16 unsigned
 * Output:	sr0	power, lsb=0.01 dB, -48.17 to 69.23 dB
 *
  ************************************************************************}
log10_dec:
	call log10;			{ 4.12 log10 output in sr1 }
	{ The log routine produces logs*4096.  To convert to dB*100 we
	  to divide by 4.096.  This is 1000/2^12:   }
	mx0 = sr1;
	my0 = 1000;
	mr = mx0*my0 (SS);
	sr = ashift mr1 by -13 (hi);
	sr = sr or lshift mr0 by -13 (lo);
	rts;

{***************************  DUMP_INTRGRT()  ***************************
 *
 * A reset function to clear integrator power.
 * 1547 instruction count for N_DIV_2=512
 * log_power() has this same fcn built in.
 *
 ************************************************************************}
dump_intrgrt:
	i0=^power_lo;	l0=0;
	i1=^power_med;	l1=0;
	i2=^power_hi;	l2=0;
	m1=1;

	cntr=N_DIV_2;
	ax0=0;
	do di_loop until ce;
	    dm(i0, m1) = ax0;		{ Dump integrator: power_lo }
	    dm(i1, m1) = ax0;		{   & power_med   }
di_loop:    dm(i2, m1) = ax0;		{   & power_hi and increment index }
	rts;

.ENDMOD;
