.MODULE LMS;
.pagelength 59;

{************************************************************************
 *
 *          EZ-KIT Lite DSP Program for DSP-10 2-meter Transciever
 *		   LMS Denoise and Auto-notch Module
 *                            LMS.DSP
 *
 * NOTICE: Copyright (C) 1996, 1997, 1998, 1999 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm
 *
 * These Noise-Reduction and Auto-Notch LMS routines are based
 * on the work of Johan Forrer, KC7WW, Monroe OR,
 * and borrows heavily from Johan's ASP.DSP work.  See QEX Sept 1996.
 * This, in turn, must credit D. Hershberger, W9GR, and Dr. S. Reyer, WA9VNJ.
 * See QST and QEX for Sept 1992.
 * Also based on routines from Analog Devices Applications
 * Using the ADSP 2100 Family, Vol 1 and 2.
 *
 * 18 Oct 00 Worked over notch routines to include as working. Added delay
 *           to input of Denoise. RSL
 * 29 Nov 00 Made D1_taps programmable, up to 61. RSL
 *
 ****************************************************************************}

.const D1_TAPS=61;		{ Denoise delay line/filter size }
.const AUT_TAPS=32;		{ Autonotch delay line/filter size }
.const DELAY_TAPS=4;		{ Denoise delay line }
.const DELAY_TAPS_AUT=63;	{ Autonotch delay line }
.entry LMS_init, do_LMS, do_autonotch;

.global beta; 			{ beta factor for denoising                  }
.global denoise_length;
.global denoise_lengthm1;
.global beta_aut;  		{ beta factor for autonotch                  }
.global notch_delay;
.global decay;			{ decay factor for denoiser                  }
.global decay_aut;		{ decay factor for autonotcher               }

{------------STORAGE FOR ADAPTIVE COEFFICIENTS-------------------------------}
.VAR/PM/CIRC coeff[AUT_TAPS];
.VAR/PM/CIRC coeff1[D1_TAPS];

{----------------------------------------------------------------------------}
{                     LMS working storage		                     }
{----------------------------------------------------------------------------}
.VAR/DM/RAM fir_out;	{ FIR output                                 	     }
.VAR/DM/RAM sample; 	{ input sample                               	     }
.VAR/DM/RAM beta  ; 	{ beta factor for denoising                          }
.VAR/DM/RAM denoise_length;   { Delay line length }
.var/dm/ram denoise_lengthm1; { Delay line length minus 1 }
.VAR/DM/RAM beta_aut;  	{ beta factor for autonotch                          }
.VAR/DM/RAM e_beta;
.VAR/DM/RAM e_beta1;	{ for autonotching }
.VAR/DM/RAM notch_delay; { Delay line length TBA }
.VAR/DM/RAM decay ;	{ decay factor for denoiser                          }
.VAR/DM/RAM decay_aut;	{ decay factor for autonotcher                       }
.VAR/DM/RAM r_ar1 ;	{ pointer into coeff. array                  	     }
.VAR/DM/RAM Error ;
.VAR/DM/RAM r_ar2;	{ pointer into coeff. array }
.var/dm/ram i0_ptr;	{ LMS FIR pointer for denoise }
.var/dm/ram i0_ptr_aut;	{ LMS FIR pointer for autonotch }
.var/dm/ram i3_ptr;	{ Delay line pointer for denoise }
.var/dm/ram i3_ptr_aut;	{ Delay line pointer for autonotch }
.var/dm/ram m1_store;	{ Protect those used by the fft   }
.var/dm/ram i3_store, m3_store, l3_store;
.var/dm/ram m5_store;

{----------------------------------------------------------------------------}
{                     LMS circular buffer storage		             }
{----------------------------------------------------------------------------}
.VAR/DM/RAM/CIRC data[AUT_TAPS];	{ Circular data delay line }
.VAR/DM/RAM/CIRC data1[D1_TAPS];	{ Second delay line }
.var/dm/ram/circ D[DELAY_TAPS];		{ Input delay line for denoise }
.var/dm/ram/circ d_aut[DELAY_TAPS_AUT];	{ Input delay line for autonotch }

{------------------- Initialization------------------------------------------}
{ Beta values will be adjusted by control  program, so anything here is OK: }
LMS_init:
	AX0=0x1000;		{ Denoise Beta }
	DM(beta) =AX0;
	ax0=61;
	dm(denoise_length)=ax0;
	ax0=60;
	dm(denoise_lengthm1)=ax0;
	ax0=0x7F63;	  	{ Denoise Decay 7F63=0.9952 }
	DM(decay)=AX0;

	AX0=0x0A3D;		{ 0.080 Beta }
	DM(beta_aut)=AX0;
	ax0=31;
	dm(notch_delay)=ax0;
	AX0=0x7EB8;	  	{ 0.99  Decay }
	DM(decay_aut)=AX0;

	i4=^coeff1;		{ Denoising coefficient pointer }
	dm(r_ar1)=i4;
	i4=^coeff;		{ Denotching coefficient pointer }
	dm(r_ar2)=i4;

	i0=^data1;  		{ i0, Denoise }
	dm(i0_ptr)=i0;
	i0=^data; 		{ i0, Autonotch }
	dm(i0_ptr_aut)=i0;
	i0=^D; 			{ i3, Denoise }
	dm(i3_ptr)=i0;
	i0=^d_aut;		{ i3, Autonotch }
	dm(i3_ptr_aut)=i0;

	rts;

{----------------------------------------------------------------------------}
{                       Denoiser processing				     }
{  Input sample is in mr1, result goes back in mr1.			     }
{  Enter with i0, m0, l0, i4, m4, l4 available.  Protect m1, i3, l3, m5 }
{----------------------------------------------------------------------------}
do_LMS:	dm(sample)=mr1;			{ Save input sample }

	dm(m1_store)=m1;
	dm(i3_store)=i3; dm(l3_store)=l3;
	dm(m5_store)=m5;		{ Protect background fft, etc. }
	{ All required i, m and l regs are now available }
	i0=dm(i0_ptr); m0=1; l0=dm(denoise_length);  { Set up registers }
	m1=0;
	i3=dm(i3_ptr); l3=DELAY_TAPS;	{ Input delay line }

	{ Decay one coefficient only: }
	m4=1; m5=0;
	my0=dm(decay); 			{ Load decay value }
	i4=dm(r_ar1); l4=dm(denoise_length);	{ Load coefficient, length }
	mx0=pm(i4,m5);			{ (m5=0) }
	mr=mx0*my0(SS);    		{ decay * coeff }
	pm(i4,m4)=mr1;		  	{ Save decayed coeff, m4=1 }
	dm(r_ar1)=i4;			{ Save updated coeff, pointer }

	{ Tweak the coefficients: }
	i4=^coeff1;
	my0=dm(e_beta);
	mx0=dm(i0,m0);			{ Get data from delay line }
	cntr=dm(denoise_lengthm1);	{ Over all coefficients }
        do tweak until ce;
	    mr=mx0*my0(SS), mx0=dm(i0, m0), ay1=pm(i4, m5);
					{ Delay-line data * e_beta }
	    ar=mr1+ay1;			{ New coeff = old + e_beta*data }
tweak:	    pm(i4, m4)=ar; 		{ Save new coeff, m4=1 }

	{ Execute the FIR filter: }
        i4=^coeff1;
	mr=0, mx0=dm(i0,m0), my0=pm(i4,m4);
	cntr=dm(denoise_lengthm1);	{ Over all coefficients }
        do conv until ce;
conv:	    mr=mr+mx0*my0(ss), mx0=dm(i0,m0), my0=pm(i4,m4);
        mr=mr+mx0*my0(rnd);
        if mv sat mr;
        dm(fir_out) = mr1;             	{ Save FIR result}

	ax0=mr1;			{ Now, compute error }
	ay0=DM(sample);
	ar=ay0-ax0;			{ Error	= D - Y }
	my0=dm(beta);
	mr=ar*my0(ss);			{ e_beta = Error * Beta }
	dm(e_beta)=mr1;			{ save new e_beta }

	{ Denoise requires a short delay line ahead of the LMS FIR line
	  to prevent the acceptance of the correlated signals due to
	  preceeding LPF. This is done with the i3 pointer:  }
	ar=dm(sample);	     
	dm(i3, m0)=ar;	   		{ (m0=1) New data point to delay line }
	ar=dm(i3, m1);	   		{ (m1=0) Get old sig from delay line }
	dm(i0, m0)=ar;			{ (m0=1) Put it into the LMS FIR line }
	dm(i0_ptr)=i0;			{ save pointers for next time }
	dm(i3_ptr)=i3;

	mr1=dm(fir_out);		{ De-noised output signal data point }

	{ Now restore the index regs that are not 0 or 4 }
	m1=dm(m1_store);
	i3=dm(i3_store); l3=dm(l3_store);
	m5=dm(m5_store);
	rts;				{ Return value in mr1 }

{------------------------- Auto-Notch Processing ----------------------------}
{  Input sample is in mr1, result goes back in mr1.			     }
{  Enter with i0, m0, l0, i4, m4, l4 available. Protect m1, i3, l3, m5       }
{----------------------------------------------------------------------------}
do_autonotch:
	dm(m1_store)=m1;
	dm(i3_store)=i3; dm(l3_store)=l3;
	dm(m5_store)=m5;
	{ All required i, m and l regs are now available }

	i0=dm(i0_ptr_aut); m0=1; l0=AUT_TAPS;             { set up registers }
	m1=0;
	i3=dm(i3_ptr_aut); l3=DELAY_TAPS_AUT;
	m4=1; l4=AUT_TAPS;
	m5=0;

	dm(sample)=mr1;			{ Save input sample }

	{ Decay one coefficient only }
	MY0=DM(decay_aut); 		{ load decay value                   }
	  			        { to work must be circular!!!!!      }
	i4=DM(r_ar2);			{ load coefficient                   }
	MX0=PM(i4,m5);			{ (m5=0)                             }
	MR=MX0*MY0(SS);    		{ decay * coeff                      }
	PM(i4,m4)=MR1;		  	{ (m4=1) save decayed coeff.         }
	DM(r_ar2)=i4;			{ save updated coeff. pointer        }

	{ Tweak the coefficients }
	i4=^coeff;
	MY0=DM(e_beta1); MX0=DM(i0,m0);
	CNTR=AUT_TAPS-1;
        DO tweak1 UNTIL CE;
       		MR=MX0*MY0(SS);		{ delay-line data * e_beta           }
		AY1=PM(i4, m5);		{ (m5=0) next coeff.                 }
		MX0=DM(i0, m0);		{ next data                          }
		AR=MR1+AY1;
tweak1:		PM(i4, m4)=AR; 		{ (m4=1) save coeff.                 }

	{ Execute the FIR filter }
        i4=^coeff;			{ set up adresses                    }
	CNTR=AUT_TAPS-1;
	MR=0, MX0=DM(i0, m0), MY0=PM(i4, m4);
        DO conv1 UNTIL CE;
conv1:   	MR=MR+MX0*MY0(SS), MX0=DM(i0, m0), MY0=PM(i4, m4);
        MR=MR+MX0*MY0(RND);
        IF MV SAT MR;			{ mr1 has FIR output }

	{ Compute Error, ie New data point - FIR output data point }
	AY0=DM(sample);			{ Get new input data point }
	AR=AY0-mr1;			{ Error	= D - Y }
        DM(Error) = ar;              	{ Save Error for notch signal out }
	MY0=DM(beta_aut);
	MR=AR*MY0(SS);			{ e_beta = Error * Beta }
	DM(e_beta1)=MR1;		{ save new e_beta }

	{ Wrap up and return }
	{ Autonotch requires a 63 tap delayed signal to be put into X array  }
	AR=DM(sample);	     
	dm(i3, m0)=ar;	   		{ (m0=1) update 63 tap input delay   }
	ar=DM(i3, m1);	   		{ (m1=0) Auto notch                  }
	DM(i0, m0)=AR;			{ (m0=1)                             }
	dm(i0_ptr_aut)=i0;		{ save registers }
	dm(i3_ptr_aut)=i3;

	{ Now restore the index regs that are not 0 or 4 }
	m1=dm(m1_store);
	i3=dm(i3_store); l3=dm(l3_store);
	m5=dm(m5_store);

	mr1=dm(Error);			{ output signal }	  
	rts;
.ENDMOD;
