.module/ram/abs=0X3800      UMONITOR;
.pagelength  59;                    { Works for .LST files with 550C }

.include    <system.k>;
.include    <monconst.k>;

{************************************************************************
 *
 *          EZ-KIT Lite DSP Program for DSP-10 2-meter Transciever
 *			UMON.DSP Version 2.1
 *                   Monitor for Non DSP-10 use
 *
 * NOTICE: Copyright (C) 2000, 2001, 2004 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This license does not purport to restrict, in any way, public domain
 * programs, where they are used as a basis for routines here.
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm
 *
 * This monitor portion fothe DSP-10 is very close to that supplied by
 * AD for their EZ-KIT Lite. It has been combined into a single file and
 * modified to allow it to be jumped to as a result of either the $$$ or
 * $OK commands. Some changes have been made to correct errors in DSP
 * test routines. Thanks to Analog Devices for making this available.
 *
 *  The original AD source codes include the following files:
 *  INTVEC  .DSP : interrupt vector table
 *  UART    .DSP : UART simulation routine
 *  MAIN    .DSP : main module
 *  SELFTEST.DSP : power up self test routine
 *  UTILITY .DSP : send a line of ASCII text through serial line
 *  COMMAND .DSP : command loop and command interpreter
 *  CODEC   .DSP : 1847 codec initialization and interrupt handling
 *  TONE    .DAT : 32 samples of one complete cycle sine wave
 *  CONSTANT.K   : constants
 *
 * Ver U2.0 Initial rewrite  11 Nov 01. RSL
 *    Changed names of globals to hide them.
 *    Cut down sign on and beeps.
 *    Added dis sec_reg to ena sec_reg at CODEC.
 *    Corrected reset register test.
 *    Added routines to initialize DSP-10 hardware shift registers
 * Ver U2.1 Modified to leave PF7 high, after programming RSL 12 Mar 04
 *
 ************************************************************************}

.entry  mon_start;

.var/pm/ram/abs=0x3fc0     intsVec [0x30];      { do not change this address }

.var/dm    ok[9];
.var/dm    bad[11];
.var/dm    aliveCnt;
.var/dm    usrRan;
.var/dm    temp;

.var/dm  flag_rx_stop_yet;          { Flag: a rx stop bit is not pending }
.var/dm  flag_rx_off;               { Flag: the receiver is turned off }
.var/dm  timer_tx_ctr;              { div by 3, timer running @ 3x baudrate }
.var/dm  timer_rx_ctr;              { div by 3, timer running @ 3x baudrate }
.var/dm  user_tx_buffer;            { UART tx reg loaded by user b4 UART xmit }
.var/dm  user_rx_buffer;            { UART rx reg read by usr aftr word rcvd }
.var/dm  internal_tx_buffer;        { Formatted for serial word, adds
                                      start & stop bits.  'user_tx_buffer' is
                                      copied here before xmission }
.var/dm  internal_rx_buffer;
.var/dm  bits_left_in_tx;           { num bits in tx buff (not yet clkd out)}
.var/dm  bits_left_in_rx;           { num bits to be rcvd (not yet clkd in) }
.var/dm  mon_char_ready;            { !0 when input character ready }
.var/dm  led_cntr;                  { led blinker counter at 3 * baud rate }
.var/dm  led_cntr_top;              { led blinker counter top }
{ signon was in DM }
.var/pm   signon [81];
.init signon:
'A','n','a','l','o','g',' ','D','e','v','i','c','e',
's',' ','A','D','S','P','-','2','1','8','1',' ','E','Z','-','K',
'I','T',' ','L','I','T','E',' ','M','o','n',' ','V','e','r',' ',
'U','2','.','1',0;
{Analog Devices ADSP-2181 EZ-KIT LITE Mon Ver U2.1}

{   DO NOT CHANGE THE ADDRESS or ORDER OF THE FOLLOWING BLOCK OF DECLARATIONS.
    THIS IS SO THAT THEY GO TO A KNOWN LOCATION IN DM.
}
.var/dm/ram/abs=0x3e00   mon_rx_buf[3];      { Status + L data + R data }
.var/dm/ram/abs=0x3e04   mon_tx_buf[3];      { Cmd + L data + R data    }
.var/dm/ram/abs=0x3e08   ptr_to_get;     { pointer to routines, see .init }
.var/dm/ram/abs=0x3e09   mon_flag_rx_no_word; { demo prog check for byte
                                                from host: 0=char ready,
                                                           1=not ready }
.var/dm/ram/abs=0x3e0a   ptr_to_out;     { pointer to routines, see .init }
.var/dm/ram/abs=0x3e0b   mon_flag_tx_ready;  { demo prog check if send to
                                        host is done: 0: transmitting, 1: done}
.var/dm/ram/abs=0x3e0c   mon_flag_rx_ready;  { demo program check to see if
                                           currently: 0: receiving, 1: done}
.var/dm/ram/abs=0x3e0d   ptr_to_codec_ini;  { pointer to routines, see .init }
.var/dm/ram/abs=0x3e10   init_cmds[13];

{ Following added at L1.1 to access to monitor RSL }
.var/dm/ram/abs=0x3ec0   ptr2mon_usr_rtn;
.var/dm/ram/abs=0x3ec1   ptr2mon_init_uart;
.var/dm/ram/abs=0x3ec2   ptr2mon_turn_rx_on;
.var/dm/ram/abs=0x3ec3   ptr2mon_codecBeep;
.var/dm/ram/abs=0x3ec4   ptr2lastCmdOK;
.var/dm/ram/abs=0x3ec5   ptr2led_cntr_top;
.var/dm/ram/abs=0x3ec6   ptr2mon_cmd_dispatch;

.var/dm     mon_stat_flag;
.var/dm     mon_codecBeep;
.var/dm     selfTstRst;             { self test result. }
.var/dm     save2n3;

.var/pm     tone [32];
.init tone: <tone.dat>;

.const   L_CTRL_REG_VALUE=0xC002;   {EZ-KIT values }
.const   R_CTRL_REG_VALUE=0xC102;

{******************************************************************************
 *
 *  These codes are executed immediately after reset.
 *
 *  After a few registers are set the self test routine is called, which in
 *  turn calls codec initialization routines.  Then the UART simulation
 *  initialization routine and UART start routine are called.  At last it
 *  beeps and send 8 lines of ASCII text to the UART port, save the interrupt
 *  vectors and jumps to the command parser.
 *
 *****************************************************************************}

mon_start:
        dis ints;                       { Returned here from User Program }
	ifc=b#0000000011111111;         { Impending be gonth }
	nop;

        imask = b#0000000000;
            {     |||||||||` | timer
                  ||||||||`- | SPORT1 rec or IRQ0
                  |||||||`-- | SPORT1 trx or IRQ1
                  ||||||`--- | BDMA
                  |||||`---- | IRQE
                  ||||`----- | SPORT0 rec
                  |||`------ | SPORT0 trx
                  ||`------- | IRQL0
                  |`-------- | IRQL1
                  `--------- | IRQ2
            }

        icntl = b#00010;
            {     ||||`- | IRQ0: 0=level, 1=edge
                  |||`-- | IRQ1: 0=level, 1=edge
                  ||`--- | IRQ2: 0=level, 1=edge
                  |`---- 0
                  `----- | IRQ nesting: 0=disabled, 1=enabled
            }

        mstat = b#1000000;
            {     ||||||`- | Data register bank select
                  |||||`-- | FFT bit reverse mode (DAG1)
                  ||||`--- | ALU overflow latch mode, 1=sticky
                  |||`---- | AR saturation mode, 1=saturate, 0=wrap
                  ||`----- | MAC result, 0=fractional, 1=integer
                  |`------ | timer enable
                  `------- | GO MODE
            }

        ax0 = b#0000000000000000;   dm (System_Control_Reg) = ax0;
            {   +-/!||+-----/+-/- | program memory wait states
                |  !|||           | 0
                |  !|||           |
                |  !||`---------- 0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !|`----------- SPORT1 1=serial, 0=FI, FO, IRQ0, IRQ1, SCLK
                |  !`------------ SPORT1 1=enabled, 0=disabled
                |  `============= SPORT0 1=enabled, 0=disabled
                `---------------- 0
                                  0
                                  0
            }

{============== INITIALIZATION ============================================}
{ For the U2.x version of the monitor, we arrived at this point by a jump from
  the user program, such as UHF3 for the DSP-10. This could have stack problems
  or other errors and needs to be reset to a known state. This follows: }

        dm(save2n3) = ar;		{ The 2nd and 3rd bytes following '$' }

        {   pop stacks; in case user program did not empty stacks }
	
        pop sts;       pop sts;       pop sts;       pop sts;
	imask=0;
        mstat = b#1000000;

        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;
        pop cntr, pop pc, pop loop;

     	call init_dm;

        l7 = 0; m7=1;

        { Arrange 0x39DE for uartXX below }
        nop; nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;

        { First two calls:
              (1) move the DSP-10 interrupt vectors to 3FC0 area and
              (2) replaces them with the monitor interrupt vectors.}
        call uhf_copy_vec;              { Added vU2.0 for entry from appl }
        call mon_copy_vec;              { Added vU2.0 for entry from appl }
        call mon_dsp10_init;
        ena ints;
        call selftest;                  { self test, includes codecInit }

	{ mon_init_uart must follow selftest to get LED rate }
        call mon_init_uart;             { Init UART routine, incl ena timer }
        call mon_turn_rx_on;            { turn on uart }

        ar = BEEP_10_mS;                { Do a beep }
        dm(mon_codecBeep) = ar;

        ar = dm(save2n3);
	none = pass ar;
        if ne jump mon_cmd_dispatch;    { Going to partial mon for $$$ & $OK }

        { Sign on banner back to PC }
        i7 = ^signon;
        call mon_sendLinePM;              { send line to host }

        jump cmdProc;                   { Go wait for a command }

{******************************************************************************
 *
 *  CODEC stops IRQ and auto-buffering; codecInit require afterward.
 *
 ****************************************************************************}

mon_codecStop:
        imask = 0;

        ax0 = 0;
        dm (SPORT0_Autobuf) = ax0;
        dm (SPORT0_Control_Reg) = ax0;

        ax0 = dm (System_Control_Reg);
        ay0 = b#1110111111111111;    { Mask to disable SPORT0 }
        ar = ax0 and ay0;
        dm (System_Control_Reg) = ar;
        rts;			/* This needs to end up at 0X3808 for EZFAST */

{*****************************************************************************
 *
 *  CODEC initialization routine.
 *
 ***************************************************************************}
mon_codecInit:
        {   shut down sport 0 during re-initialization }
        call mon_codecStop;

        ifc = b#00000011111111;         { clear pending interrupt }
        nop;

        i5 = ^mon_rx_buf;
        l5 = %mon_rx_buf;
        i6 = ^mon_tx_buf;
        l6 = %mon_tx_buf;
        i7 = ^init_cmds;
        l7 = %init_cmds;

        m5 = 1;

{================== S E R I A L   P O R T   #0   S T U F F ==================}
        ax0 = b#0000110011010111;   dm (SPORT0_Autobuf) = ax0;
            {   |||!|-/!/|-/|/|+- receive autobuffering 0=off, 1=on
                |||!|  ! |  | +-- transmit autobuffering 0=off, 1=on
                |||!|  ! |  +---- | receive m?
                |||!|  ! |        | m5
                |||!|  ! +------- ! receive i?
                |||!|  !          ! i5
                |||!|  !          !
                |||!|  +========= | transmit m?
                |||!|             | m5
                |||!+------------ ! transmit i?
                |||!              ! i6
                |||!              !
                |||+============= | BIASRND MAC biased rounding control bit
                ||+-------------- 0
                |+--------------- | CLKODIS CLKOUT disable control bit
                +---------------- 0
            }

        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
            {   RFSDIV = SCLK Hz/RFS Hz - 1 }
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
            {   SCLK = CLKOUT / (2  (SCLKDIV + 1) }
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
            {   multichannel
                ||+--/|!||+/+---/ | number of bit per word - 1
                |||   |!|||       | = 15
                |||   |!|||       |
                |||   |!|||       |
                |||   |!||+====== ! 0=right just, 0-fill; 1=right just, signed
                |||   |!||        ! 2=compand u-law; 3=compand A-law
                |||   |!|+------- receive framing logic 0=pos, 1=neg
                |||   |!+-------- transmit data valid logic 0=pos, 1=neg
                |||   |+========= RFS 0=ext, 1=int
                |||   +---------- multichannel length 0=24, 1=32 words
                ||+-------------- | frame sync to occur this number of clock
                ||                | cycle before first bit
                ||                |
                ||                |
                |+--------------- ISCLK 0=ext, 1=int
                +---------------- multichannel 0=disable, 1=enable
            }
            {   non-multichannel
                |||!|||!|||!+---/ | number of bit per word - 1
                |||!|||!|||!      | = 15
                |||!|||!|||!      |
                |||!|||!|||!      |
                |||!|||!|||+===== ! 0=right just, 0-fill; 1=right just, signed
                |||!|||!||+------ ! 2=compand u-law; 3=compand A-law
                |||!|||!|+------- receive framing logic 0=pos, 1=neg
                |||!|||!+-------- transmit framing logic 0=pos, 1=neg
                |||!|||+========= RFS 0=ext, 1=int
                |||!||+---------- TFS 0=ext, 1=int
                |||!|+----------- TFS width 0=FS before data, 1=FS in sync
                |||!+------------ TFS 0=no, 1=required
                |||+============= RFS width 0=FS before data, 1=FS in sync
                ||+-------------- RFS 0=no, 1=required
                |+--------------- ISCLK 0=ext, 1=int
                +---------------- multichannel 0=disable, 1=enable
            }

        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
            {   ^15          00^   transmit word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
            {   ^31          16^   transmit word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
            {   ^15          00^   receive word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;
            {   ^31          16^   receive word enables: channel # == bit # }


{============== S Y S T E M   A N D   M E M O R Y   S T U F F ==============}
        ax0 = b#0001000000000000;       { OR in to enable SPORT0  }
        ay0 = dm (System_Control_Reg);
        ar = ax0 or ay0;
        dm (System_Control_Reg) = ar;

{******************************************************************************
 *  ADSP 1847 Codec intialization
 *****************************************************************************}

        { Enable SPORT0 transmit interrupt }
        imask = b#0001000000;
            {     |||||||||` | timer
                  ||||||||`- | SPORT1 rec or IRQ0
                  |||||||`-- | SPORT1 trx or IRQ1
                  ||||||`--- | BDMA
                  |||||`---- | IRQE
                  ||||`----- | SPORT0 rec
                  |||`------ | SPORT0 trx
                  ||`------- | IRQL0
                  |`-------- | IRQL1
                  `--------- | IRQ2
            }

        {   set flag, disable tx ints }
        ax0 = 0;
        dm(mon_stat_flag) = ax0;

{   The following codes make sure the address register wrap around is
    in sync with the frame and verifies the codec is responding correctly. }
mon_again:
        ax0 = 0;                    { tx 0 to left and right }
        dm (mon_tx_buf + 1) = ax0;
        dm (mon_tx_buf + 2) = ax0;
        ax0 = dm (init_cmds + 11);  { codec to switch to 32 slot per frame }
        dm (mon_tx_buf) = ax0;

        ax0 = dm (i6, m5);          { start interrupt }
        tx0 = ax0;

        idle;

        ax0 = 0x8dac;               { write ac to Digital mix control }
        dm (mon_tx_buf) = ax0;

        idle;
        ax0 = 0x2d00;               { readback digital mix control }
        dm (mon_tx_buf) = ax0;
        idle;
        idle;                       { for 32 slot, readback is in next frame }
        ax0 = dm (mon_rx_buf);
        ay0 = 0x2dac;
        none = ax0 - ay0;
        if ne jump mon_again;           { if not reading back, not in sync }


        ax0 = 0x8d50;               { write 50 to Digital mix control }
        dm (mon_tx_buf) = ax0;

        idle;
        ax0 = 0x2d00;               { readback digital mix control }
        dm (mon_tx_buf) = ax0;
        idle;
        idle;                       { for 32 slot, readback is in next frame }
        ax0 = dm (mon_rx_buf);
        ay0 = 0x2d50;
        none = ax0 - ay0;
        if ne jump mon_again;           { if not reading back, not in sync }


        {   clear flag }
        ax0 = 1;
        dm(mon_stat_flag) = ax0;

        ax0 = dm (init_cmds + 12);  { codec to switch to 32 slot per frame }
        dm (mon_tx_buf) = ax0;

check_init:
        idle;
        ax0 = dm (mon_stat_flag);       { wait for entire init }
        none = pass ax0;            { buffer to be sent to }
        if ne jump check_init;      { the codec            }

check_aci1:
        idle;
        ay0 = 2;
        ax0 = dm (mon_rx_buf);          { once initialized, wait for codec }
        ar = ax0 and ay0;           { to come out of autocalibration }
        if eq jump check_aci1;      { wait for bit set }

check_aci2:
        idle;
        ax0 = dm (mon_rx_buf);          { wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci2;

        ay0 = 0xbf3f;               { unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (mon_tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);   { unmute right DAC }
        ar = ax0 AND ay0;
        dm (mon_tx_buf) = ar;
        idle;

        ax0 = 0xa000;               { control word to clear over-range flags }
        dm (mon_tx_buf) = ax0;
        idle;

        ifc = b#00000011111111;     { clear any pending interrupt }
        nop;

        imask = b#0000100000;       { enable rx0 interrupt }
            {     |||||||||` | timer
                  ||||||||`- | SPORT1 rec or IRQ0
                  |||||||`-- | SPORT1 trx or IRQ1
                  ||||||`--- | BDMA
                  |||||`---- | IRQE
                  ||||`----- | SPORT0 rec
                  |||`------ | SPORT0 trx
                  ||`------- | IRQL0
                  |`-------- | IRQL1
                  `--------- | IRQ2
            }
        rts;

{*******************************  ISR's  ************************************
 *
 *  Interrupt service routines
 *
 *  Receive interrupt used for loopback
 *
 ***************************************************************************}

mon_input_samples:
        ena sec_reg;                { use shadow register bank }

        ax1 = dm (mon_rx_buf + 1);      { loopback inputs to outputs }
        mx1 = dm (mon_rx_buf + 2);
        dm (mon_tx_buf + 1) = ax1;
        dm (mon_tx_buf + 2) = mx1;

        { Puts out a tone if beep enabled }
        ar = dm(mon_codecBeep);
        ar = ar - 1;
        if eq jump mon_is1;             { Not enabled }
        dm(mon_codecBeep) = ar;         { Continue count }
        ay0 = %tone - 1;		{ 0b0000000000011111 }
        af = ar and ay0;
        ar = ^tone;
        ar = ar + af;
        ax0 = i7;                       { Save old i7 }
        i7 = ar;
#if 0
	{ Old when tone was in DM. This is all kinda awkward! }
        ar = dm (i7, m7);               { Get current tone value }
#endif
        ar = pm (i7, m7);               { Get current tone value }
        i7 = ax0;                       { Restore i7 }
        dm (mon_tx_buf + 1) = ar;       { Output tone }
        dm (mon_tx_buf + 2) = ar;
mon_is1:
        dis sec_reg;                    { Added Rev L1.1 }
        rti;

{------------------------------------------------------------------------------
 -
 -  transmit interrupt used for Codec initialization
 -
 -----------------------------------------------------------------------------}
mon_next_cmd:
        ena sec_reg;
        ax0 = dm (i7, m5);          { fetch next control word and }
        dm (mon_tx_buf) = ax0;      { place in transmit slot 0    }
        ax0 = i7;
        ay0 = ^init_cmds;
        ar = ax0 - ay0;
        if gt jump nc_rti;          { rti if more control words still waiting }
        ax0 = 0xaf00;               { else set done flag and }
        dm(mon_tx_buf) = ax0;       { remove MCE if done initialization }
        ax0 = 0;
        dm(mon_stat_flag) = ax0;    { reset status flag }
nc_rti: dis sec_reg;
        rti;

{******************************************************************************
 *
 *  Send null terminated string pointed to by i7 and LF/CR.  ASCII is in
 *  lower 8-bit.
 *
 *  Rev L1.1 Bob Larkin 10 Feb 00 - Made location in pm seg.
 *
 *****************************************************************************}
mon_sendLine:
        l7 = 0;
        m7 = 1;
msl1:  ax1 = dm (i7, m7);              { Get a char from buffer }
        none = pass ax1;                { Is it end of string? }
        if eq jump linefeed;            { Yes, so append CRLF }
        call mon_out_char_ax1;
        jump msl1;                      { Go get next char }

linefeed:
        ax1 = 13;                       { CR }
        call mon_out_char_ax1;
        ax1 = 10;                       { LF }
        call mon_out_char_ax1;
        rts;

{ Rev L1.1 Bob Larkin 10 Feb 00 - Cut tones to 10 mS click, made global
           names unique to monitor. Made loc seg.
}

{*************************  COMMAND  *****************************************
 *
 *  This command parser accepts these commands:
 *
 *  1.  Beep
 *      received:$$$
 *      send    :ok[led count][alive called count][selftest hi][selftest lo]
 *
 *
 *      [led count] is the lower 8 bits of a counter incremented at 28800 rate.
 *      [alive called count] is incremented by one each time alive is called.
 *      [selftest hi][selftest lo] is the selftest result
 *      beep
 *
 *  2.  Alive inquiry
 *      received:$OK
 *      send    :ok[led count][alive called count][selftest hi][selftest lo]
 *
 *      [led count] is the lower 8 bits of a counter incremented at 28800 rate.
 *      [alive called count] is incremented by one each time alive is called.
 *      [selftest hi][selftest lo] is the selftest result
 *
 *  3.  upload DM
 *      received:$UD[hi start][lo start][hi len][lo len]
 *      send    :[hi][lo]...[hi sum][lo sum]$!
 *
 *  4.  upload PM
 *      received:$UP[hi start][lo start][hi len][lo len]
 *      send    :[hi][mi][lo]...[hi sum][mi sum][lo sum]$!
 *
 *  5.  download DM
 *      received:$DD[hi start][lo start][hi len][lo len][hi][lo]...
 *      send    :![hi sum][lo sum]
 *
 *  6.  download PM
 *      received:$DP[hi start][lo start][hi len][lo len][hi][mi][lo]...
 *      send    :![hi sum][mi sum][lo sum]
 *
 *  7.  Call subroutine
 *      received:$GO[hi address][loaddress]
 *      send    :![hi address][loaddress]
 *
 *****************************************************************************}

cmdProc:                                { Wait for first $ }
set FL1;
        idle;
        call mon_get_char_ax1;
set FL2;
        ay1 = 0x24;
        none = ax1 - ay1;
        if eq jump cmdPrmpt;            { receive command prompt $ }

        none = pass ax1;
        if eq jump cmdProc;             { skip 0x00 }

        {-- command not recognized --}
        dm (bad+9) = ax1;         { for others, send back string: you type:? }
        i7 = ^bad;
        call mon_sendLine;
        jump cmdProc;

cmdPrmpt:
        {-- now get the next two characters (will time out.) --}
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }

        none = pass ar;
        if eq jump cmdProc;     { if both zero, discard; host clearing }
        { Now the first char is in upper 8 bits of ar and second in lower. }

mon_cmd_dispatch:
        { This is the command dispatcher  }
        m7 = 1;
        l7 = 0;

        ay0 = 0x4f4b;           { ASCII for 'OK' }
        none = ar - ay0;
        if eq jump alive;

        ay0 = 0x2424;           { ASCII for '$$' }
        none = ar - ay0;
        if eq jump beep;

        ay0 = 0x5544;           { ASCII for 'UD' }
        none = ar - ay0;
        if eq jump updm;

        ay0 = 0x5550;           { ASCII for 'UP' }
        none = ar - ay0;
        if eq jump uppm;

        ay0 = 0x4444;           { ASCII for 'DD' }
        none = ar - ay0;
        if eq jump dndm;

        ay0 = 0x4450;           { ASCII for 'DP' }
        none = ar - ay0;
        if eq jump dnpm;

        ay0 = 0x474f;           { ASCII for 'GO' }
        none = ar - ay0;
        if eq jump go;

        {-- command not regconized --}
        dm (bad + 9) = ar;
        i7 = ^bad;

lastCmdBad:
        ar = BEEP_10_mS;            { beep (click) }
        dm (mon_codecBeep) = ar;
        ar = LEDRateBad;            { get fast LED blink rate value. }
        dm (led_cntr_top) = ar;     { set led counter reset value. }
        jump cmdProc;

lastCmdOK:
        ar = LEDRateGood;           { get slow LED blink rate value. }
        dm (led_cntr_top) = ar;     { set led counter reset value. }
        jump cmdProc;

{******************************************************************************
 *  received:$$$
 *  send    :ok[led count][alive called count][selftest hi][selftest lo]
 *****************************************************************************}

beep:
        ar = BEEP_10_mS;              { enable codec to beep }
        dm (mon_codecBeep) = ar;

{******************************************************************************
 *  received:$OK
 *  send    :ok[led count][alive called count][selftest hi][selftest lo]
 *****************************************************************************}
alive:
        {-- byte 2 is led counter; byte 3 is alive called count --}
        ar = 0x6f6b;                { send ok }
        call mon_out_int_ar;

        ax1 = dm (led_cntr);        { get led counter }
        call mon_out_char_ax1;          { send lower 8 bits }

        ar = dm (aliveCnt);         { get and increment aliveCnt }
        ar = ar + 1;
        dm (aliveCnt) = ar;
        ax1 = ar;
        call mon_out_char_ax1;          { send lower 8 bits }

        ar = dm(selfTstRst);       { get selftest result }
        call mon_out_int_ar;
        ar = 0x0a0d;                { send line feed/carriage return }
        call mon_out_int_ar;
        jump lastCmdOK;

{******************************************************************************
 *  received:$UD[hi start][lo start][hi len][lo len]
 *  send    :[hi][lo]...[hi sum][lo sum]$!
 *****************************************************************************}
updm:
        {   get start }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }
        i7 = ar;

        {   get len }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }

        {   the loop }
        af = pass 0;                { zero checksum }
        cntr = ar;
        do updm0 until ce;
        ar = dm (i7, m7);
        af = ar + af;               { compute checksum }
        call mon_out_int_ar;        { send to host }
updm0:  nop;

        ar = pass af;               { send checksum }
        call mon_out_int_ar;

        jump lastCmdOK;

{******************************************************************************
 *  received:$UP[hi start][lo start][hi len][lo len]
 *  send    :[hi][mi][lo]...[hi sum][mi sum][lo sum]$!
 *****************************************************************************}
uppm:
        {   get start }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }
        i7 = ar;

        {   get len }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }

        {   the loop }
        mr = 0;                     { zero checksum }
        cntr = ar;
            do uppm0 until ce;
            ay1 = pm (i7, m7);
            { compute checksum }
            sr1 = 0;                    { px in lower 8 sr0 }
            sr0 = px;
            si = ay1;
            sr = sr or lshift si by 8 (lo);    { or in upper 16 bits }
            ay0 = mr0;
            ar = sr0 + ay0;             { do a 24 bits add; hi:lo in mr1:mr0 }
            mr0 = ar;                   { number in lower 24 bits of 32 bits }
            ay0 = mr1;
            ar = sr1 + ay0 + c;         { add with carry }
            mr1 = ar;
            ar = ay1;
            call mon_out_int_ar;            { send upper 16 bits. }
            ax1 = px;
            call mon_out_char_ax1;          { send lower 8 bits. }
uppm0:      nop;

        {   compute and send 24 bits checksum }
        {   these codes are also used by download pm }
        sr = lshift mr0 by 8 (lo);  { shirt upper 16 bits into sr1 }
        sr = sr or lshift mr1 by 8 (hi);  { shirt upper 16 bits into sr1 }
        ar = sr1;
        call mon_out_int_ar;            { send upper 16 bits. }
        ax1 = mr0;                  { lower 8 bits still in mr0 }
        call mon_out_char_ax1;          { and lower 8 bits. }

        jump lastCmdOK;

{******************************************************************************
 *  received:$DD[hi start][lo start][hi len][lo len][hi][lo]...
 *  send    :![hi sum][lo sum]
 *****************************************************************************}

dndm:
        {   get start }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }
        i7 = ar;

        {   get len }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }

        {   the loop }
        af = pass 0;
        cntr = ar;
        do dndm0 until ce;
            call mon_get_int_ar_to;
            if lt jump dndmBad;         { check time out: jump if timeout }
            dm (i7, m7) = ar;
dndm0:      af = ar + af;               { compute checksum }

        {   send checksum }
        ar = pass af;               { send checksum }
        call mon_out_int_ar;

        jump lastCmdOK;

dndmBad:
        {   time out, clean up counter, loop, and PC stacks. }
        {   These codes are also used by download pm }
      { pop cntr; }         { no need to pop counter because it's empty. }
        pop pc;                     { popping relavent stacks since we }
        pop loop;                   { are jumping out of do loop. }
        jump lastCmdBad;

{******************************************************************************
 *  received:$DP[hi start][lo start][hi len][lo len][hi][mi][lo]...
 *  send    :![hi sum][mi sum][lo sum]
 *****************************************************************************}

dnpm:
        {   get start }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }
        i7 = ar;

        {   get len }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;      { check time out }

        {   the loop }
        mr = 0;
        cntr = ar;
        do dnpm0 until ce;
            call mon_get_int_ar_to;
            if lt jump dndmBad;         { check time out: share download dm codes }
            ay1 = ar;                   { save upper 16 bits in ay1 }
            call mon_get_char_ax1_to;   { get lowest 8 bits. }
            if lt jump dndmBad;         { check time out: share download dm codes }
            px = ax1;                   { load px }
            { compute checksum }
            sr1 = 0;                    { px in lower 8 sr0 }
            sr0 = px;
            si = ay1;
            sr = sr or lshift si by 8 (lo);    { or in upper 16 bits }
            ay0 = mr0;
            ar = sr0 + ay0;             { do a 24 bits add; hi:lo in mr1:mr0 }
            mr0 = ar;                   { number is in lower 24 bits of 32 bits }
            ay0 = mr1;
            ar = sr1 + ay0 + c;         { add with carry }
            mr1 = ar;
dnpm0:      pm (i7, m7) = ay1;          { save to pm }

        {   compute and send 24 bits checksum }
        jump uppm0;                 { share upload pm code }

{******************************************************************************
 *  received:$GO[hi address][loaddress]
 *  send    :![hi address][loaddress]
 *
 *  user program to jump to 0x3fff to continue monitor program.
 *
 *  before running user program:
 *  1.  dis timer
 *  2.  enable IRQ1
 *  3.  mask all ints
 *****************************************************************************}
go:
        {   get address }
        call mon_get_int_ar_to;
        if lt jump lastCmdBad;       { check time out }
        i7 = ar;
        call mon_out_int_ar;         { echo address back to host }
go1:    ar = dm (mon_flag_tx_ready); { wait until last transmit is done. }
        none = pass ar;
        if eq jump go1;

        ar = 1;                     { set user program running flag }
        dm (usrRan) = ar;

        dis timer;                  { disable timer (re-enable by init_uart) }
        imask = 0;

        ax0 = 0;
        dm (System_Control_Reg) = ax0;  {   shut down sport 0 }
        dm (SPORT0_Autobuf) = ax0;      {   turn off auto-buffering }
        ay1 = i7;                   { copy 0x3fc0..ef 0x00..2f }
        i4 = 0;                     { PC will download 0..2f to 3fc0..ef }
        l4 = 0;   m4 = 0;
        i7 = ^intsVec;
        l7 = 0;   m7 = 1;
        cntr = %intsVec;
        do usrVec1 until ce;
            ax0 = pm (i4, m4);      { current int vec to be saved }
            ax1 = px;
            ay0 = pm (i7, m4);      { downloaded int vec to move to low mem }
            {   px implicit }
            pm (i4, m7) = ay0;
            px = ax1;
usrVec1:    pm (i7, m7) = ax0;
        i7 = ay1;
        {   call USER codes }
        call (i7);                      { This allows return to mon with rts }

        {           ****** Off to User Program here *****            }

mon_usr_rtn:                            { Set up a label for jump returns }
        dis ints;                       { Returned here from User Program }
        dis timer;
        ifc = b#00000011111111;         { clear pending interrupt }
        nop;

        {   pop stacks; in case user program did not empty stacks }
        pop sts;       pop sts;       pop sts;       pop sts;
        pop cntr;      pop cntr;      pop cntr;      pop cntr;
        pop pc;        pop pc;        pop pc;        pop pc;
        pop loop;      pop loop;      pop loop;      pop loop;

        ar = 0;                    { clear user program running flag }
        dm (usrRan) = ar;

        {   restore just in case }
        l7 = 0;
        m7 = 1;                    { restore for memory transfer }

        i4 = 0;                    { restore saved int vec. }
        l4 = 0;
        i7 = ^intsVec;
        cntr = %intsVec;
        do usrVec2 until ce;
           ar = pm (i7, m7);       { downloaded int vec to move to low mem }
usrVec2:   pm (i4, m7) = ar;       {   px implicit }

        icntl = b#00010;
        mstat = b#1000000;
        imask = b#0000000001;

        ar = 1;
        dm (mon_codecBeep) = ar;

        ena ints;

        call mon_codecInit;

        call mon_init_uart;         { intialize uart routine }
        call mon_turn_rx_on;        { turn on uart }

        ar = BEEP_10_mS;            { enable codec to beep }
        dm (mon_codecBeep) = ar;

        jump lastCmdOK;

{******************************************************************************
 *  A high to low transition on flag_in signifies the start bit; it also
 *  triggers IRQ1 ISR which then turn on timer if the timer is off.  This is
 *  at most 1/3 bit period too late, but we should still catch the byte.
 *
 * (Routine not used? Starts by looking at FLAG_IN )
 *
 * RTI and RTS differ in popping status (ASTAT, MSTAT and IMASK) back to
 * their resisters (unlike PC, CNTR  or LOOP).
 * pop sts; ena timer; rts; works like rti except that ena timer will be
 * put into place. RTI would have overwritten MSTAT.  RSL 11 dec 01.
 *****************************************************************************}
mon_irq1isr:
        pop sts;
        ena timer;              { start timer now }
        rts;                    { note rts }

{**************************************************************************

   ADSP-2101 Family Software UART

    This uses FLAG_IN, FLAG_OUT and the TIMER of ADSP-2101 to interface to
    an RS-232 asynchronous serial device such as a VT100 terminal.  ex:

        ADSP-2101 FLAG_OUT ----------> AD233 ----------> RS-232 RX

        ADSP-2101  FLAG_IN <---------- AD233 <---------- RS-232 TX
                       (TIMER maintains baudrate)

    Parameters bits/word, baudrate, stopbits & parity are user-programmable.

    The operation of the transmitter setup routine is completely
    independent on the the receiver setup routine operation.  Although
    both tx and rx use the same timer as a master clock source, the
    xmitted bits need not be in sync with the received bits. The default
    state of the reciever is OFF, so the "turn_rx_on" subroutine must be
    used to enable RX.

Calling Argument:
             for autobaud load the baud constant
        dm(baud_period)=(Proc_frequency/(3*Baudrate))-1

Useful Subroutines:
        init_uart        Must be called after system reset.
        mon_get_char_ax1 Waits for RX input and returns with it in ax1.
        mon_out_char_ax1 Waits for last TX output and transmits data from ax1.
        mon_turn_rx_on   Must be called to enable the receipt of RX data.
        mon_turn_rx_off  Can be used to ignore input RX data.

Useful Flag:
        DM(mon_flag_rx_ready) If this DM location is all ones it indicates that
                            the UART is ready to rx new word. If it is zero
                            then data is being received. Can be used for xon
                            xoff flow control.

    Author:   Fares Eidi, 21-May-90, Analog Devices Inc.
    modified: Christoph D. Cavigioli, 17-Dec-90
    modified: Steven Cox, 20-Dec-91, extensive rewrite
    modified: Steven Cox, 31-Dec-91, Reset flag_rx_stop_yet to 1, Comments.
    modified: Steven Cox, 11-Feb-92, Added support for autobaud.
    modified: Philip Holdgate, 02-Apr-92, modified for Apps Note, cleaned up.

****************************************************************************}

{ UART Parameters }
.const CRYSTAL_FREQ_IN_kHZ=   16667;    { UART crystal speed related constant }
.const  tx_num_of_bits = 10;    { start bits + tx data bits + stop bits }
.const  rx_num_of_bits = 8;     { rx data bits (start&stop bits not counted)}
.const  RX_BIT_ADD = 0x0100;    { = 1<<rx_num_of_bits }
.const  TX_BIT_ADD = 0xfe00;    { = 0xffff<<(tx data bits+1) }

{   EZ-KIT: period count == 33 MHz / 3 / 9600 - 1 == 1144 }
.const  PERIOD = (CRYSTAL_FREQ_IN_kHZ * 2000 / (3 * 9600)) - 1;

{************************* Initializing Subroutine **************************}

filler:
    nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
    nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
    nop; nop; nop; nop; nop; nop; nop; nop; nop; nop;
    nop;
    { Adjust the number of nops to keep uartXX at 0X39DE }

mon_init_uart:
    ax0=0;
    dm(TSCALE)=ax0;             { decrement TCOUNT every instruction cycle }
    dm(mon_char_ready)=ax0;         { no input character ready }

    ax0 = dm (selfTstRst);      { retrieve self test result }
    ar = LEDRateBad;            { led counter top: no error, 2 Hz. }
    ay0 = LEDRateGood;
    none = pass ax0;
    if eq ar = pass ay0;        { led counter top: with error, 8 Hz. }
    dm (led_cntr_top) = ar;     { set led counter top. }
    dm (led_cntr) = ar;         { clear LED blinker counter }

uartXX:
    ax0=PERIOD;                 { autobaud not used, Needs to be at 0X39DE
					for EZFAST  }
    dm(TCOUNT)=ax0;
    dm(TPERIOD)=ax0;            { interrupts generated at 3x baudrate }

    ax0=1;
    dm(mon_flag_tx_ready)=ax0;  { set the flags showing that UART is not busy }
    dm(mon_flag_rx_ready)=ax0;
    dm(flag_rx_stop_yet)=ax0;
    dm(mon_flag_rx_no_word)=ax0;
    dm(flag_rx_off)=ax0;        { rx section off }

    set flag_out;               { UART tx output is initialized to high }
    ifc = b#00000011111111;     { clear pending interrupt }
    nop;                        { wait for ifc latency }
    { SPORT1 must be configured for FI, FO, etc:  }
    ax0 = b#0000100000000000;
    ay0 = dm (System_Control_Reg);
    ar = ax0 or ay0;            { enable SPORT1 }
    ay0 = b#1111101111111111;
    ar = ar and ay0;            { clear to enable FI, etc. }
    dm (System_Control_Reg) = ar;
    ax0=imask;
    ay0=b#0000000001;		{ Timer int enable }
    ar=ax0 or ay0;
    imask=ar;                   { enable TIMER interrupt handling }
    ena timer;                  { start timer now }
    rts;

{*************************  Stop UART Subroutine  **************************}
{ Routine not used }
stop_uart:
    dis timer;                  { start timer now }
    set flag_out;               { UART tx output is initialized to high }
    ifc = b#00000011111111;     { clear pending interrupt }
    nop;                        { wait for ifc latency }
    ar=imask;
    ay0=b#1111111110;
    ar=ar and ay0;
    imask=ar;                   { enable TIMER interrupt handling }
    rts;

{*********************  Process_a_bit (TIMER interrupt routine) *************
 *
 *  This routine is the heart of the UART.  It is called every timer
 *  interrupt (i.e. 3x baudrate).  This routine will xmit one bit at a
 *  time by setting/clearing the FLAG_OUT pin of the ADSP-2101. This routine
 *  will then test if the uart is already receiving. If not it will test
 *  flagin (rx) for a start bit and place the uart in receive mode if true.
 *  If already in receive mode it will shift in one bit at a time by reading
 *  the FLAG_IN pin.  Since the internal timer is running at 3x baudrate,
 *  bits need only be transmitted/received once every 3 timer interrupts.
 *
 *****************************************************************************}

mon_process_a_bit:
    ena sec_reg;                    { Switch to the background dreg set }

    {-- LED blinker--}
    ar = dm (usrRan);               { see if user program is running }
    none = pass ar;                 { do not blink if user program is running }
    if ne jump noled;
    ar = dm (led_cntr);             { LED blinker counter }
    ax0 = dm (led_cntr_top);        { LED blinker counter top }
    ar = ar - 1;
    if eq toggle fl1;               { toggle LED }
    if eq ar = pass ax0;            { reset counter }
    dm (led_cntr) = ar;             { save counter }
noled:

    ax0=dm(mon_flag_tx_ready); { if not in "transmit", go right to "receive" }
    none=pass ax0;
    if ne jump receiver;
               {_________Transmitter Section________}
    ay0=dm(timer_tx_ctr);           { test timer ctr to see if a bit }
    ar=ay0-1;                       { is to be sent this time around }
    dm(timer_tx_ctr)=ar;            { if no bit is to be sent }
    if ne jump receiver;            { then decrement ctr and return }

    sr1=dm(internal_tx_buffer);     { shift out LSB of internal_tx_buffer }
    sr=lshift sr1 by -1 (hi);       { into SR1.  Test the sign of this bit }
    dm(internal_tx_buffer)=sr1;     { set or reset FLAG_OUT accordingly }
    ar=pass sr0;                    { this effectively clocks out the }
    if ge reset flag_out;           { word being xmitted one bit at a time }
    if lt set flag_out;             { LSB out first at FLAG_OUT. }

    ay0=3;                          { reset timer ctr to 3, i.e. next bit }
    dm(timer_tx_ctr)=ay0;           { will be sent after 3 timer interrupts }

    ay0=dm(bits_left_in_tx);        { number of bits left to be xmitted }
    ar=ay0-1;                       { is now decremented by one, }
    dm(bits_left_in_tx)=ar;         { indicating that one is now xmitted }
    if gt jump receiver;            { if no more bits left, then ready }

    ax0=1;                          { flag is set to true indicating }
    dm(mon_flag_tx_ready)=ax0;      { a new word can now be xmitted }

               {_________Receiver Section________}
receiver:
    ax0=dm(flag_rx_off);            { Test if receiver is turned on }
    none=pass ax0;
    if ne jump ex_rti;

    ax0=dm(flag_rx_stop_yet);       { Test if finished with stop bit of }
    none=pass ax0;                  { last word or not. if finished then }
    if ne jump rx_test_busy;        { continue with check for receive. }

    ay0=dm(timer_rx_ctr);           { decrement timer ctr and test to see }
    ar=ay0-1;                       { if stop bit period has been reached }
    dm(timer_rx_ctr)=ar;            { if not return and wait }
    if ne jump ex_rti;

    ax0=1;                          { if stop bit is reached then reset }
    dm(flag_rx_stop_yet)=ax0;       { to wait for next word }
    dm(mon_flag_rx_ready)=ax0;
    dm(mon_char_ready) = ax0;       { set to 1 indicates character available }

    ax0=dm(internal_rx_buffer);     { copy internal rx buffer }
    dm(user_rx_buffer)=ax0;         { to the user_rx_buffer }

    ax0=0;                          { indicated that a word is ready in }
    dm(mon_flag_rx_no_word)=ax0;        { the user_rx_buffer }
    jump ex_rti;

rx_test_busy:
    ax0=dm(mon_flag_rx_ready);          { test rx flag, if rcvr is not busy }
    ar=pass ax0;                    { receiving bits then test for start.If it}
    if eq jump rx_busy;             { is busy, then clk in one bit at a time }

    if flag_in jump rx_exit;        { Test for start bit and return if none }

    ax0=0;
    dm(mon_flag_rx_ready)=ax0;      { otherwise, indicate rcvr is now busy }
    dm(internal_rx_buffer)=ax0;     { clear out rcv register}

    ax0=4;                          { timer runs @ 3x baud rate, so rcvr }
    dm(timer_rx_ctr)=ax0;           { will only rcv on every 3rd interrupt }
                                    { initially this ctr is set to 4.  This }
                                    { will skip the start bit and will }
                                    { allow us to check FLAG_IN at the center }
                                    { of the received data bit }
    ax0=rx_num_of_bits;
    dm(bits_left_in_rx)=ax0;
rx_exit:
    jump ex_rti;

rx_busy:
    ay0=dm(timer_rx_ctr);           { decrement timer ctr and test to see }
    ar=ay0-1;                       { if bit is to be rcvd this time around }
    dm(timer_rx_ctr)=ar;            { if not return, else receive a bit }
    if ne jump ex_rti;

rcv:            { Shift in rx bit }
    ax0=3;                          { reset the timer ctr to 3 indicating }
    dm(timer_rx_ctr)=ax0;           { next bit is 3 timer interrupts later }

    ay0=RX_BIT_ADD;
    ar=dm(internal_rx_buffer);
    if not flag_in jump pad_zero;   { Test RX input bit and }
    ar=ar+ay0;                      { add in a 1 if hi }

pad_zero:
    sr=lshift ar by -1 (lo);        { Shift down to ready for next bit }
    dm(internal_rx_buffer)=sr0;

    ay0=dm(bits_left_in_rx);        { if there are more bits left to be rcvd }
    ar=ay0-1;                       { then keep UART in rcv mode }
    dm(bits_left_in_rx)=ar;         { and return }
    if gt jump ex_rti;              { if there are no more bits then.. }

                       { That was the last bit }
    ax0=3;                          { set timer to wait for middle of the }
    dm(timer_rx_ctr)=ax0;           { stop bit }
    ax0=0;                          { flag indicated that uart is waiting }
    dm(flag_rx_stop_yet)=ax0;       { for the stop bit to arrive }
ex_rti:
    dis sec_reg;
    rti;

{********************** Invoke_UART_transmit subroutine *******************
 *
 *  This is the first step in the transmit process.  The user has now
 *  loaded 'user_tx_buffer' with the ascii code and has also invoked
 *   this routine.
 *
 *************************************************************************}

invoke_UART_transmit:
    ax0=3;                          { initialize the timer decimator ctr }
    dm(timer_tx_ctr)=ax0;           { this divide by three ctr is needed }
                                    { since timer runs @ 3x baud rate }

    ax0=tx_num_of_bits;             { this constant is defined by the }
    dm(bits_left_in_tx)=ax0;        { user and represents total number of }
                                    { bits including stop and parity }
                                    { ctr is initialized here indicating }
                                    { none of the bits have been xmitted }
    sr1=0;
    sr0=TX_BIT_ADD;                 { upper bits are hi to end txmit with hi }
    ar=dm(user_tx_buffer);          { transmit register is copied into  }
    sr=sr or lshift ar by 1 (lo);   { the internal tx reg & left justified }
    dm(internal_tx_buffer)=sr0;     { before it gets xmitted }

    ax0=0;                          { indicate that the UART is busy }
    dm(mon_flag_tx_ready)=ax0;
    rts;

{********************  Get an input character  ******************************}
mon_get_char_ax1:
        ax0=dm(mon_flag_rx_no_word);
        none=pass ax0;
        if ne jump mon_get_char_ax1;   { if no rx word input, then wait }
get_char_ax1x:
        toggle fl1;                    { toggle LED }
        dm(mon_char_ready) = ax0;      { clear character ready flag }

        ax1=dm(user_rx_buffer);         { get received ascii character }
        ax0=1;
        dm(mon_flag_rx_no_word)=ax0;        { word was read }
        rts;

{******************** Get an input character with time out *****************
 *
 * output:         ax1     (all 16 bits set if timeout)
 * modifies:       ax0, af
 * time out and return 0xffff after 500 mS.
 * (15000 ticks / 3 == 5000 bps ~= 5000 / 9600  ~= .5 S)
 *
 *****************************************************************************}

mon_get_char_ax1_to:
        ax1 = ar;                       { preserve ar }
        ar = 30000;                     { 3 ticks per bit; 9600 bps, time out }
                                        { at 1 sec ==> 9600 * 3 == 30000 }
get_char_ax1_to1:
        idle;                           { wait for timer interrupt }
        ar = ar - 1;
        if eq jump get_char_ax1_to2;    { time out now }

        ax0=dm(mon_flag_rx_no_word);
        none=pass ax0;
        if ne jump get_char_ax1_to1;   { if no rx word input, then wait }
        ar = ax1;                       { restore ar }
        jump get_char_ax1x;             { read the character. }

get_char_ax1_to2:
        ar = ax1;                       { restore ar }
        ax1 = 0xffff;
        rts;

{***************** Get a 16-bit integer with time out **********************
 *
 * output:         ar
 * modifies:       ax0, ax1, si, sr0, sr1, ay0
 * astat:          ge if ok; lt if timed out
 *
 *****************************************************************************}

mon_get_int_ar_to:
        {   get hi 8 bits }
        call mon_get_char_ax1_to;
        none = pass ax1;            { check time out }
        if lt rts;
        si = ax1;
        sr = lshift si by 8 (lo);   { shift to high byte }
        ay0 = sr0;

        {   get lo 8 bits }
        call mon_get_char_ax1_to;
        none = pass ax1;            { check time out }
        if lt rts;
        ar = ax1 or ay0;            { or in low byte }
        none = pass ax1;            { ax1 is >= 0; set ge for ok return code }
        rts;

{********************** Output a character *********************************
 *
 * input:          ax1 (preserved)
 * modifies:       ax0, sr1, sr0, ar
 *
 *****************************************************************************}

mon_out_char_ax1:
        ax0=dm(mon_flag_tx_ready);
        none=pass ax0;
        if eq jump mon_out_char_ax1; { if tx word out still pending, then wait}
        toggle fl1;                { toggle LED }
        dm(user_tx_buffer)=ax1;
        call invoke_UART_transmit;      { send it out }
        rts;

{********************* Output a 16-bit int *********************************
 *
 * input:          ar (preserved)
 * modifies:       ax0, ax1, sr1, sr0, ay0
 *
 *****************************************************************************}

mon_out_int_ar:
        ay0 = ar;                  { save lo 8 bits }
        sr = lshift ar by -8 (lo); { shift hi 8 bits to lo 8 bits }
        ax1 = sr0;
        call mon_out_char_ax1;     { send hi 8 bits }
        ax1 = ay0;
        call mon_out_char_ax1;     { send lo 8 bits }
        rts;

{********************** Enable the RX section ****************************
 *
 * modifies:       ax0
 *  memory :       dm(flag_rx_off)
 *
 *****************************************************************************}

mon_turn_rx_on:
        ax0=0;
        dm(flag_rx_off)=ax0;
        rts;

{********************** Disable the RX section ****************************
 * Not used
 *
 *****************************************************************************}
mon_turn_rx_off:
        ax0=1;
        dm(flag_rx_off)=ax0;
uartYY:
        rts;

{**********************  SELFTEST  *******************************************
 *
 *  Performs the following tests:
 *      1.  reset register states test
 *      2.  DSP register test
 *      3.  do loop test
 *      4.  on-chip program and data memory test
 *      5.  1847 codec test
 *
 *
 *  Result bit codes :
 *
 *  0x0001 = reset memory mapped register state test failed.
 *  0x0002 = DSP registers test failed.
 *  0x0004 = do loop test failed.
 *  0x0008 = program and data memory test failed
 *  0x0010 = codec initialization failed
 *
 *****************************************************************************}

.const  PM_UTOP=0x37FF;
.const  DM_UTOP=0x3DFF;

selftest:
        ar = 0;
        dm (selfTstRst) = ar;

#if 0
{This was the ADI approach, apparently because variations in different
 versions of the ADSP2181 have different startup values.  )

{=== reset state test =============================================}
       { Checking register contents agains known reset state value }

        ax0 = 0x0000;   ay0 = astat;
        none = ax0 - ay0;
        if ne jump test1bad;

        ax0 = 0x0054;   ay0 = sstat;
        none = ax0 - ay0;
        if ne jump test1bad; 

        ax0 = 0x0000;   ay0 = dm (System_Control_Reg);
        none = ax0 - ay0;
        if ne jump test1bad;

        ax0 = 0x7fff;   ay0 = dm (DM_Wait_Reg);
        none = ax0 - ay0;
        if ne jump test1bad;

        ay0 = dm (SPORT0_Control_Reg);
        none = pass  ay0;
        if ne jump test1bad;

        ax0 = 0xf003;   ay0 = dm (SPORT0_Autobuf);
        ar  = ax0 and ay0;
        none = pass ar;
        if ne jump test1bad;

        ax0 = 0x7fff;   ay0 = dm (SPORT1_Control_Reg);
        ar  = ax0 and ay0;
        none = pass ar;
        if ne jump test1bad;

        ax0 = 0x7b00;   ay0 = dm (PFTYPE);  { Corrected from 7F00 - RSL }
        none = ax0 - ay0;
        if ne jump test1bad;
#endif

        { By virtue of these codes being run, BDMA registers must be ok }

        jump test1ok;
test1bad:
        ax0 = 0x0001;
        ay0 = dm (selfTstRst);
        ar = ax0 or ay0;
        dm (selfTstRst) = ar;
test1ok:

{=== DSP registers test ====================================================}
{   reading and writing DSP registers                                       }
        ax0 = 0xffff;
        call copyReg;
        ay0 = si;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0x3fff;
        ay0 = l7;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0xff;
        ay0 = px;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0xffff;
        ay0 = sb;
        none = ax0 - ay0;
        if ne jump test2bad;

        ax0 = 0x5555;
        call copyReg;
        ay0 = si;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0x1555;
        ay0 = l7;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0x55;
        ay0 = px;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0xfff5;
        ay0 = sb;
        none = ax0 - ay0;
        if ne jump test2bad;

        ax0 = 0xaaaa;
        call copyReg;
        ay0 = si;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0x2aaa;
        ay0 = l7;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0xaa;
        ay0 = px;
        none = ax0 - ay0;
        if ne jump test2bad;
        ax0 = 0x0a;
        ay0 = sb;
        none = ax0 - ay0;
        if ne jump test2bad;

        ax0 = 0x0000;
        call copyReg;
        ay0 = si;
        none = pass ay0;
        if ne jump test2bad;
        ay0 = l7;
        none = pass ay0;
        if ne jump test2bad;
        ay0 = px;
        none = pass ay0;
        if ne jump test2bad;
        ay0 = sb;
        none = pass ay0;
        if ne jump test2bad;

{-- test pcovlay }
{   better not swap our world in memory away.
    DO NOT CHANGE PCOVLAY AS IT SWAP OUT MONITOR CODES.
        ar = 0;
        pmovlay = ar;
        ay0 = pmovlay;
        none = ar - ay0;
        if ne jump test2bad;

        ar = 1;
        pmovlay = ar;
        ay0 = pmovlay;
        none = ar - ay0;
        if ne jump test2bad;

        ar = 2;
        pmovlay = ar;
        ay0 = pmovlay;
        none = ar - ay0;
        if ne jump test2bad;
}

{-- test dcovlay }
        ar = 0;
        dmovlay = ar;
        ay0 = dmovlay;
        none = ar - ay0;
        if ne jump test2bad;

        ar = 1;
        dmovlay = ar;
        ay0 = dmovlay;
        none = ar - ay0;
        if ne jump test2bad;

        ar = 2;
        dmovlay = ar;
        ay0 = dmovlay;
        none = ar - ay0;
        if ne jump test2bad;

        jump test2ok;
test2bad:
        ax0 = 0x0002;
        ay0 = dm (selfTstRst);
        ar = ax0 or ay0;
        dm (selfTstRst) = ar;
test2ok:
        ar = 0;
        pmovlay = ar;
        dmovlay = ar;

{=== Do loop control test ==================================================}
{   Do four level deep do loop and check number of executions               }
        ar = 0;
        cntr = 15;
        do lb1 until ce;
            cntr = 15;
            do lb2 until ce;
                cntr = 15;
                do lb3 until ce;
                    cntr = 15;
                    do lb4 until ce;
                        ar = ar + 1;
lb4:                nop;
lb3:            nop;
lb2:        nop;
lb1:    nop;
        ay0 = 50625;
        none = ar - ay0;
        if ne jump test3bad;

        jump test3ok;
test3bad:
        ax0 = 0x0004;
        ay0 = dm (selfTstRst);
        ar = ax0 or ay0;
        dm (selfTstRst) = ar;
test3ok:

{=== program and data memory test ==========================================}
{ This test is skipped for ver U2.x as the areas for dm/pm are linker assigned
  RSL 12 Dec 01.
}
{   write known patterns to unused program and data program and verify      }
{   array pm_user and dm_user occupy unused memories                        }
{   pm }
  JUMP test4ok;   { verU2.X }
       { i4 = 0x0030; verU2.x }                   { ^pm_user; }
        m4 = 1;
        cntr = PM_UTOP-0x0030;          { %pm_user; }
        do lb10 until ce;
            ar = i4;
lb10:       pm (i4, m4) = ar;

        i4 = 0x0030;                    { ^pm_user; }
        cntr = PM_UTOP-0x0030;          { %pm_user; }
        do lb11 until ce;
            ar = i4;
            ay0 = pm (i4, m4);
            none = ar - ay0;
            if eq jump lb11;
lb10a:      { pop cntr; }     { no need to pop counter because it's empty. }
            pop pc;                     { popping relavent stacks since we }
            pop loop;                   { are jumping out of do loop. }
            jump test4bad;
lb11:       nop;

        {   dm }
        i4 = 0;                         { ^dm_user; }
        cntr = DM_UTOP;                 { %dm_user; }
        do lb12 until ce;
            ar = i4;
lb12:       dm (i4, m4) = ar;

        i4 = 0;                         { ^dm_user; }
        cntr = DM_UTOP;                 { %dm_user; }
        do lb13 until ce;
            ar = i4;
            ay0 = dm (i4, m4);
            none = ar - ay0;
            if ne jump lb10a;           { jump to pop stacks }
lb13:       nop;

        jump test4ok;

test4bad:
        ax0 = 0x0008;
        ay0 = dm (selfTstRst);
        ar = ax0 or ay0;
        dm (selfTstRst) = ar;
test4ok:

{=== 1847 test =============================================================}
{   initialize 1847 codec and verify all codec register values              }
        ena ints;
        call mon_codecInit;
        cntr = 400;
        do lb31 until ce;
        idle;
lb31:   nop;
        ax0 = 0x2000;   dm (mon_tx_buf) = ax0;   idle;
        ax0 = 0x2100;   dm (mon_tx_buf) = ax0; idle;
        ax0 = 0x2200;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);  {dm (0x2000) = ax0;}
        ay0 = 0x2002;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2300;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2001) = ax0;}
        ay0 = 0x2102;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2400;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2002) = ax0;}
        ay0 = 0x2288;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2500;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2003) = ax0;}
        ay0 = 0x2388;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2600;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2004) = ax0;}
        ay0 = 0x2488;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2700;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2005) = ax0;}
        ay0 = 0x2588;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2800;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2006) = ax0;}
        ay0 = 0x2600;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2900;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2007) = ax0;}
        ay0 = 0x2700;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2a00;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2008) = ax0;}
        ay0 = 0x285b;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2c00;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x2009) = ax0;}
        ay0 = 0x2909;   none = ax0 - ay0; if ne jump test5bad;
        ax0 = 0x2d00;   dm (mon_tx_buf) = ax0; idle;
        ax0 = dm (mon_rx_buf);{dm (0x200a) = ax0;}
        ay0 = 0x2a00;   none = ax0 - ay0; if ne jump test5bad;
                idle;   ax0 = dm (mon_rx_buf);{dm (0x200b) = ax0;}
        ay0 = 0x2c40;   none = ax0 - ay0; if ne jump test5bad;
                idle;   ax0 = dm (mon_rx_buf);{dm (0x200c) = ax0;}
        ay0 = 0x2d00;   none = ax0 - ay0; if ne jump test5bad;
        jump test5ok;
test5bad:
        ax0 = 0x0010;
        ay0 = dm (selfTstRst);
        ar = ax0 or ay0;
        dm (selfTstRst) = ar;
test5ok:
        ax0 = 0xaf00;
        dm (mon_tx_buf + 0) = ax0;

        { turn off transmit interrupt }
        ax0 = imask;
        ay0 = b#1110111111;             { AND to turn off SPORT0 TX }
        ar = ax0 and ay0;
        imask = ar;
        ay0 = dm (selfTstRst);
        rts;				{ Return form Self Test }

{********************  DSP-10 Hardware Initialization  *******************

  The following is specific for the DSP-10. It programs the
  two 74HC595 shift registers to a known state at startup and thus prevents
  having the transceiver start in transmit. It then returns the PFFLAG's
  to their normal 'input' startup condition. Note that during the few
  microseconds that the latch routine is running, PF7 is set for output. In
  the unlikely event that the EZKIT is used with hardware for which this
  is a problem, this version of the monitor should not be used.

  FLAG Pin Hardware Usage (part):
    FL0	Out	Data for all serial devices
    FL2	Out	Clock for all serial devices
    PF7	Out	Latch Enable for shift reg and 126 MHz synthesizer

  Bob Larkin, 17 Feb 00
  Made separate routine 12 Dec 01 RSL
}
mon_dsp10_init:
        ax0=b#0001000100000100;
        {     |||||||||||||||`-- Not Used
              ||||||||||||||`--- Not Receive
              |||||||||||||`---- Not Transmit
              ||||||||||||`----- Power Amp
              |||||||||||`------ Ant Relay
              ||||||||||`------- E1 (Extra)
              |||||||||`-------- E2
              ||||||||`--------- E3
              |||||||`---------- -18 dB R_ATT1
              ||||||`----------- -12 dB R_ATT1
              |||||`------------  -6 dB R_ATT1
              ||||`-------------   0 dB R_ATT1
              |||`-------------- -18 dB R_ATT2
              ||`--------------- -12 dB R_ATT2
              |`----------------  -6 dB R_ATT2
              `-----------------   0 dB R_ATT2                      }
	reset fl2;			{ Be sure clock starts low }
	cntr=16;
	do mi2 until ce;
	   ar=tstbit 15 of ax0;		{ Transfer this bit to shift reg }
	   sr0=ax0;
	   sr=lshift sr0 by 1 (lo);	{ Move bits over, status unaffected }
	   ax0=sr0;			{ Save shifted data }
	   if eq jump mi0;
	   set fl0;			{ Data line to 1 }
	   jump mi1;
mi0:	   reset fl0;			{ Data line to 0 }
mi1:	   call mon_delay10u;
	   set fl2;			{ Clock line to a 1--shift the bit in }
	   call mon_delay10u;
	   reset fl2;			{ Clock fall--ready for next time }
	   call mon_delay10u;
mi2:       nop;
	{ Latch the 16 bits into the shift regs by a pulse on PF7 }
	{ First, make PF7 an output pin: }
        ax0=b#0111101110000000;
        {             `--------- Prog Flag 7, PFTYPE=Output       }
	dm(PFTYPE)=ax0;
	ax0=dm(PFDATA);			{ Get ready to work on it }
	ar=clrbit 7 of ax0;		{ Make sure bit 7 is low to start }
	dm(PFDATA)=ar;
	call mon_delay10u;
	ar=setbit 7 of ax0;		{ Set bit 7 to latch data }
	dm(PFDATA)=ar;
	call mon_delay10u;
	{ Rev 12 March 2004 - Leave PF7 high to minimize false enables }
     {	ar=clrbit 7 of ax0;  }
     {	dm(PFDATA)=ar;       }
	{ Now reset the programmable flags to input, since we don't know
	  what they might be used for. }
        ax0=b#0111101100000000;
        {             ````````-- Prog Flags, PFTYPE=Input  }
	dm(PFTYPE)=ax0;
        rts;

{*************************************************************************
 *  This sub-routine is used by the DSP register test.  AX0 is set to the
 *  desireable pattern and passed through all registers to be tested.
 *  The last register in the chain is si (16 bits,) l7 (14 bits,) px (8
 *  bits,) and sb (5 bits).
 *
 *  16 bits:
 *      ax0, ax1, ay0, ay1, af, ar
 *      mx0, mx1, my0, my1, mr0, mr1, mf
 *      si, sr0, sr1
 *  14 bits;
 *      i0-7, m0-7, l0-7
 *  8 bits:
 *      mr2, se, px
 *  5 bits:
 *      sb
 *  2 bits:
 *      pmovlay, dmovlay
 *
 ************************************************************************}
copyReg:
        ax1 = ax0;  ar  = ax1;  mr0 = ar;  mr1 = mr0;
        sr0 = mr1;  sr1 = sr0;  ay0 = sr1; ay1 = ay0;
        af  = pass ay1;  ar = pass af;
        mx0 = ar;  mx1 = mx0;  my0 = mx1;  my1 = my0;  si  = my1;
        i0 = si;  i1 = i0;  i2 = i1;  i3 = i2;
        i4 = i3;  i5 = i4;  i6 = i5;  i7 = i6;
        m0 = i7;  m1 = m0;  m2 = m1;  m3 = m2;
        m4 = m3;  m5 = m4;  m6 = m5;  m7 = m6;
        l0 = m7;  l1 = l0;  l2 = l1;  l3 = l2;
        l4 = l3;  l5 = l4;  l6 = l5;  l7 = l6;
        mr2 = l7; se = mr2; px = se;  sb = px;
        rts;

init_dm:
        { .init ok: 'o', 'k', 0, 0, 0, 0, 0, 0, 0; }
        ax0=79;                    { 'O' }
        dm(ok)=ax0;
        ax0=75;                     { 'K' }
        dm(ok+1)=ax0;
        ax0=0;
        dm(ok+2)=ax0;
        dm(ok+3)=ax0;
        dm(ok+4)=ax0;
        dm(ok+5)=ax0;
        dm(ok+6)=ax0;
        dm(ok+7)=ax0;
        dm(ok+8)=ax0;
        { .init bad: 'y','o','u',' ','t','y','p','e',':', 0, 0; }
        ax0=89;                    { 'Y' }
        dm(bad)=ax0;
        ax0=111;                   { 'o' }
        dm(bad+1)=ax0;
        ax0=117;                   { 'u' }
        dm(bad+2)=ax0;
        ax0=32;                    { ' ' }
        dm(bad+3)=ax0;
        ax0=116;                   { 't' }
        dm(bad+4)=ax0;
        ax0=121;                   { 'y' }
        dm(bad+5)=ax0;
        ax0=112;                   { 'p' }
        dm(bad+6)=ax0;
        ax0=101;                   { 'e' }
        dm(bad+7)=ax0;
        ax0=58;                    { ':' }
        dm(bad+8)=ax0;
        ax0=0;
        dm(bad+9)=ax0;
        dm(bad+10)=ax0;

#if 0
{   These values are also checked in self-test;  they must agree. }
.init init_cmds:
        L_CTRL_REG_VALUE,
                    {   EZ-KIT: 0xc00f
                        Left input control reg
                        b7-6: 0=left line 1
                              1=left aux 1
                              2=left line 2
                              3=left line 1 post-mixed loopback
                        b5-4: res
                        b3-0: left input gain x 1.5 dB
                    }
        R_CTRL_REG_VALUE,
                    {   EZ-KIT: 0xc10f
                        Right input control reg
                        b7-6: 0=right line 1
                              1=right aux 1
                              2=right line 2
                              3=right line 1 post-mixed loopback
                        b5-4: res
                        b3-0: right input gain x 1.5 dB
                    }
        0xc288,     {
                        left aux 1 control reg
                        b7  : 1=left aux 1 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc388,     {
                        right aux 1 control reg
                        b7  : 1=right aux 1 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc488,     {
                        left aux 2 control reg
                        b7  : 1=left aux 2 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc588,     {
                        right aux 2 control reg
                        b7  : 1=right aux 2 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc680,     {
                        left DAC control reg
                        b7  : 1=left DAC mute
                        b6  : res
                        b5-0: attenuation x 1.5 dB
                    }
        0xc780,     {
                        right DAC control reg
                        b7  : 1=right DAC mute
                        b6  : res
                        b5-0: attenuation x 1.5 dB
                    }
        0xc85b,     {
                        data format register
                        b7  : res
                        b5-6: 0=8-bit unsigned linear PCM
                              1=8-bit u-law companded
                              2=16-bit signed linear PCM
                              3=8-bit A-law companded
                        b4  : 0=mono, 1=stereo
                        b0-3: 0=  8.
                              1=  5.5125
                              2= 16.
                              3= 11.025
                              4= 27.42857
                              5= 18.9
                              6= 32.
                              7= 22.05
                              8=   .
                              9= 37.8
                              a=   .
                              b= 44.1
                              c= 48.
                              d= 33.075
                              e=  9.6
                              f=  6.615
                       (b0) : 0=XTAL1 24.576 MHz; 1=XTAL2 16.9344 MHz
                    }
        0xc909,     {
                        interface configuration reg
                        b7-4: res
                        b3  : 1=autocalibrate
                        b2-1: res
                        b0  : 1=playback enabled
                    }
        0xca00,     {
                        pin control reg
                        b7  : logic state of pin XCTL1
                        b6  : logic state of pin XCTL0
                        b5  : master - 1=tri-state CLKOUT
                              slave  - x=tri-state CLKOUT
                        b4-0: res
                    }
        0xcc40,     {
                        miscellaneous information reg
                        b7  : 1=16 slots per frame, 0=32 slots per frame
                        b6  : 1=2-wire system, 0=1-wire system
                        b5-0: res
                    }
        0xcd00;     {
                        digital mix control reg
                        b7-2: attenuation x 1.5 dB
                        b1  : res
                        b0  : 1=digital mix enabled
                    }
#endif

        ax0=L_CTRL_REG_VALUE;
        dm(init_cmds)=ax0;
        ax0=R_CTRL_REG_VALUE;
        dm(init_cmds+1)=ax0;
        ax0=0xc288;
        dm(init_cmds+2)=ax0;
        ax0=0xc388;
        dm(init_cmds+3)=ax0;
        ax0=0xc488;
        dm(init_cmds+4)=ax0;
        ax0=0xc588;
        dm(init_cmds+5)=ax0;
        ax0=0xc680;
        dm(init_cmds+6)=ax0;
        ax0=0xc780;
        dm(init_cmds+7)=ax0;
        ax0=0xc85b;
        dm(init_cmds+8)=ax0;
        ax0=0xc909;
        dm(init_cmds+9)=ax0;
        ax0=0xca00;
        dm(init_cmds+10)=ax0;
        ax0=0xcc40;
        dm(init_cmds+11)=ax0;
        ax0=0xcd00;
        dm(init_cmds+12)=ax0;

        { .init aliveCnt: 0x2f; }
        ax0=0x2f;
        dm(aliveCnt)=ax0;
        { .init usrRan: 0; }            { 1 if user program is running. }
        ax0=0;
        dm(usrRan)=ax0;

        {.init mon_tx_buf:   0xc000, 0x0000, 0x0000;  Initially set MCE }
        ax0=0xC000;
	dm(mon_tx_buf)=ax0;
        ax0=0x0000;
	dm(mon_tx_buf+1)=ax0;
	dm(mon_tx_buf+2)=ax0;

        {.init mon_codecBeep: 1; }
        ax0=1;
        dm(mon_codecBeep)=ax0;

        ax0=^mon_get_char_ax1_to;
        dm(ptr_to_get)=ax0;
        ax0=^mon_out_char_ax1;
        dm(ptr_to_out)=ax0;
        ax0=^mon_codecInit;
        dm(ptr_to_codec_ini)=ax0;

        ax0=^mon_usr_rtn;
        dm(ptr2mon_usr_rtn)=ax0;
        ax0=^mon_init_uart;
        dm(ptr2mon_init_uart)=ax0;
        ax0=^mon_turn_rx_on;
        dm(ptr2mon_turn_rx_on)=ax0;
        ax0=^mon_codecBeep;
        dm(ptr2mon_codecBeep)=ax0;
        ax0=^lastCmdOK;
        dm(ptr2lastCmdOK)=ax0;
        ax0=^led_cntr_top;
        dm(ptr2led_cntr_top)=ax0;
        ax0=^mon_cmd_dispatch;
        dm(ptr2mon_cmd_dispatch)=ax0;

        ar = LEDRateGood;           { get slow LED blink rate value. }
        dm(led_cntr_top) = ar;      { set led counter reset value. }
	dm(led_cntr) = ar;

        ar=1;
        dm(mon_flag_rx_no_word) = ar;   { Byte waiting: 1=No   (3e09) }
        dm(mon_flag_tx_ready) = ar;     { Serial to PC: 1=No   (3e0b) }
	dm(mon_flag_rx_ready) = ar;     { Serial from PC: 1=No (3e0c) }

        rts;

{ DELAY10U()  -  Do nothing for 10 microseconds, used to init DSP-10 }
mon_delay10u:
	cntr=330;
	do mdly0 until ce;
mdly0:	   nop;
	rts;

{ The following copies the 48 instructions from pm=0 to the interrupt
  storage area at 3FC0. This is in preparation for running the monitor when
  entry has come from the DSP-10. The px register conveys the least-significant
  8 bits in the moves.   RSL  Rev 2.0 }
uhf_copy_vec:
        i4 = 0;                     { Source location }
        l4 = 0;
        m4 = 1;
        i7 = 0x3FC0;                { Destination location }
        l7 = 0;
        cntr = 48;                  { Number of instructions }
        do uhfvec until ce;
            ax0 = pm(i4,m4);        { Get instr to be moved, increment i4 }
uhfvec:     pm(i7,m4) = ax0;        { Move with increment of i7, px carries 8 }
        rts;

{ The following copies the 48 instructions from high PM to the interrupt
  vector area at pm=0. This is in preparation for running the monitor when
  entry has come from the DSP-10. The px register conveys the least-significant
  8 bits in the moves.   RSL  Rev 2.0 }
mon_copy_vec:
        i4 = ^mon_int_vec_sav;      { Source location }
        l4 = 0;
        m4 = 1;
        i7 = 0;                     { Destination location }
        l7 = 0;
        cntr = 48;                  { Number of instructions }
        do monvec until ce;
            ax0 = pm(i4,m4);        { Get instr to be moved, increment i4 }
monvec:     pm(i7,m4) = ax0;        { Move with increment of i7, px carries 8 }
        rts;

{ This code can be placed in 0x0000..0x002f, the interrupt vectors area,
  when the monitor program is to be run.  It is saved in here, but of course
  never run here.  RSL U2.0 }
mon_int_vec_sav:
        jump mon_start;          rti; rti; rti;     {00: reset }
        rti;                     rti; rti; rti;     {04: IRQ2 }
        rti;                     rti; rti; rti;     {08: IRQL1 }
        rti;                     rti; rti; rti;     {0c: IRQL0 }
        ar = dm(mon_stat_flag);                     {10: SPORT0 tx }
        ar = pass ar;
        if eq rti;
        jump mon_next_cmd;
        jump mon_input_samples;  rti; rti; rti;     {14: SPORT1 rx }
        rti;                     rti; rti; rti;     {18: IRQE }
        rti;                     rti; rti; rti;     {1c: BDMA }
 {Interestingly, the next item does not seem to be used - Just FLAG_IN }
        jump mon_irq1isr;        rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;                     rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
        jump mon_process_a_bit;  rti; rti; rti;     {28: timer }
        rti;                     rti; rti; rti;     {2c: power down }

{******************************************************************************
 *
 *  Send null terminated string in PM pointed to by i7 and LF/CR.  ASCII is in
 *  lower 8-bit.
 *
 *  Rev U2.0 Bob Larkin 8 Dec 01 -  Same as mon_sendLine() except from PM.
 *
 *****************************************************************************}
mon_sendLinePM:
        l7 = 0;
        m7 = 1;
mslp1:  ax1 = pm (i7, m7);              { Get a char from buffer }
        none = pass ax1;                { Is it end of string? }
        if eq jump linefeedp;           { Yes, so append CRLF }
        call mon_out_char_ax1;
        jump mslp1;                     { Go get next char }

linefeedp:
        ax1 = 13;                       { CR }
        call mon_out_char_ax1;
        ax1 = 10;                       { LF }
        call mon_out_char_ax1;
        rts;

.endmod;
